// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//	
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/O_Ants2006/O_Ants2006_swarmrtm.cc,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: O_Ants2006 swarm agent system
//       Holds code for both swarm node and swarm agent functionality.
//       Handles multiple pheroids.
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-11-07


#include <stdio.h>

#include "O_Ants2006_swarmrtm.h"
#include "delay.h"
#include <vector>
#include <algorithm>
#include <values.h>

#define T_INFINITY_APPROX 3000
#define STORE_PORTS_BEHAVIOR 0 //(Oriol) Decide the behavior of the agents, 1=>comportament de ports :: 0=>comportament de ids
#define TWO_MEASURE_BEHAVIOR 0 //(Oriol) IMPORTANT:if TWO_PHERO_BEHAVIOR IT'S ENABLED, THIS MUST BE DISABLED
#define TWO_PHERO_BEHAVIOR 0   //(Oriol) Decide the behavior of the system, 1=>two pheros per link  ::0=>one phero per link
#define BETA_QUEUES 0.95	   //(Oriol) Decide how many paths with queue have to be found to change pheromone
#define LIMIT_PHEROS 0.95	   //(Oriol) Decide when to change from one pheromone to the other one
#define MAX_OCUPATION 0.5	   //(Oriol) Decide when the second pheromone (or measure)  is used (ocupation + queue length). NOTE: always < 0.999999

// Tcl interface 
static class O_Ants2006_SwarmRtMClass : public TclClass {
public:
	O_Ants2006_SwarmRtMClass() : TclClass("RtModule/Swarm/O_Ants2006") {}
	TclObject* create(int argc, const char*const* argv) {
		return (new O_Ants2006_SwarmRtM());
	}
}class_O_Ants2006_SwarmRtM;


O_Ants2006_SwarmRtM::O_Ants2006_SwarmRtM() : SwarmRtModule()
{  

	// Set swarm type
	myswarmtype_ = SWT_O_Ants2006;

	trace_phero_ = false;
	trace_pheros_.clear();
	if(TWO_PHERO_BEHAVIOR==1){
	trace_phero2_ = false;
	trace_pheros2_.clear();}

	backtrack_rate_estimate_ = 0;
	cost_actual_=0;
	cost_actual2_=0;
	last_backtrack_time_ = 0;
	forward_rate_estimate_ = 0;
	last_forward_time_ = 0;
	last_forward_agentpacket_ = NULL;
	forward_interarrival_sum_ = 0;
	backtrack_interarrival_sum_ = 0;
	local_boost_ = false;
	min_replica_ = 0;
	ind_queues_=0;
	ultim_temps_=0;
}

int O_Ants2006_SwarmRtM::command(int argc, char const *const *argv) {
	// Process commands fired at TCL-level

	// Access to tcl layer
	Tcl& tcl = Tcl::instance(); 

	if (argc == 2) {

		if (strcmp(argv[1], "get-ref") == 0) {
			// Return pointer to this object
			tcl.resultf("%p",this);
			return TCL_OK;
		}
		
	} else if (argc == 3) {
		
		if (strcmp(argv[1], "trace-phero") == 0) {
			if (strcmp(argv[2], "none") == 0) {
				// Disable traceing of pheromones
				trace_phero_ = false;
				trace_pheros_.clear();
				if(TWO_PHERO_BEHAVIOR==1)
				{
					trace_phero2_ = false;
					trace_pheros2_.clear();
				}
				return TCL_OK;
			}

			if (strcmp(argv[2], "all") == 0) {
				// Enable traceing of all pheromones
				trace_phero_ = true;
				trace_pheros_.clear();
				if(TWO_PHERO_BEHAVIOR==1)
				{
					trace_phero2_ = true;
					trace_pheros2_.clear();
				}
				return TCL_OK;
			}
			
			// Get and store phero ids to trace
			trace_phero_ = true;
			trace_pheros_.clear();
			if(TWO_PHERO_BEHAVIOR==1)
			{
				trace_phero2_ = true;
				trace_pheros2_.clear();
			}
			// A comma separated list of phero id is expexted
			char idlist[strlen(argv[2])+1];
			int phero, phero2;
			strncpy(idlist, argv[2], strlen(argv[2])+1);
			char* id_idx = strtok(idlist, ",");
			while (id_idx) {
				if (sscanf(id_idx, "%d", &phero) == 1)
					trace_pheros_.insert(phero);
				if(TWO_PHERO_BEHAVIOR==1)
				{
					if (sscanf(id_idx, "%d", &phero2) == 1)
						trace_pheros2_.insert(phero2);
				}
				id_idx = strtok(NULL, ",");
				
			}
			return TCL_OK;
		}

		if (strcmp(argv[1], "local-boost") == 0) {
			if (strcmp(argv[2], "off") == 0) {
				// Disable local boost
				local_boost_ = false;
				return TCL_OK;
			}

			if (strcmp(argv[2], "on") == 0) {
				// Enable local boost
				local_boost_ = true;
				return TCL_OK;
			}
		}
			
		if (strcmp(argv[1], "min-replica") == 0) {
			// Set min num of agents in a local boost
			if (sscanf(argv[2], "%d", &min_replica_) == 1)
				return TCL_OK;
			tcl.add_errorf("O_Ants2006_SwarmRtM::command: Invalid min replica value '%s'.", argv[2]);
			return TCL_ERROR;
		}

	} else  if (argc == 4) {
		/* DOESN'T WORK :(
		if (strcmp(argv[1], "delete-route") == 0) {
			
			// Check if any elements in entry2neighbour map are to be removed
			Connector* entry = (Connector*)TclObject::lookup(argv[3]);
			if (entry2neighbour_.equal_range((NsObject*)entry) !=  pair< multimap<NsObject*, Neighbour*>::iterator,
				multimap<NsObject*, Neighbour*>::iterator >(entry2neighbour_.end(), entry2neighbour_.end()) ) {
				// Elements are to be removed, i.e. a route/link to a neighbour has disappeared

				if (local_boost_ )
					// Initiate local boost of agents
					local_boost(entry);
			}

			// Continue to run parent method...
		}
		*/
	}   
	
	return (SwarmRtModule::command(argc, argv));
}

void O_Ants2006_SwarmRtM::expire(Event* e)
  // Timeout handler
  // ... continue to implement swarm agent behaviour
{

	// Prepare access to swarm agent state in packet (p_ is current packet)
	swarmagent_ = (O_Ants2006_data*)p_->userdata();
	
	// Access headers
	hdr_cmn* ch = hdr_cmn::access(p_);
	hdr_ip* ip = hdr_ip::access(p_);

	// I m p l e m e n t   s w a r m   a g e n t   b e h a v i o u r

	// Short behaviour description:
	// - Visits new nodes until destination node or a dead end is reached, i.e. all neighbours have been visited.
	// - Destination can be any node including the source node. If dest = source, a TSP route is searched for.
	// - If dead end and a neigbour is destination node, goes to destination. Hence given a fully meshed network, it finds an Hamiltonian path.
	// - If dead end and destination is not neighbour, select next node uniformly among neigbours. i.e. revisit.
	// - Only elite O_Ants2006 update pheromones, however the temperature for all ants regulates the elite set

	switch (swarmagent_->state()) {
	case SWS_IDLE:
		// Agent is at home node ready to start forward seach
		swarmagent_->state() = SWS_FORWARDSEARCH;
		swarmagent_->birthtime() = now();
		swarmagent_->go_second() = 0; // (Oriol)
		swarmagent_->go_first() = 0; //(Oriol)
		swarmagent_->from1st() = 0; //(Oriol)
		swarmagent_->L_path()=0;
		swarmagent_->L_path2()=0;

		if(TWO_PHERO_BEHAVIOR==1)//(Oriol)
		{
			if(ind_queues_ <= LIMIT_PHEROS)
			{
				swarmagent_->go_second()=0;
				swarmagent_->from1st()=0;
			}else{
				swarmagent_->go_second()=0/*1*/;
				swarmagent_->from1st()=1;
			}

		}


		if (swarmagent_->behaviour() == O_Ants2006_datapacket) {
			// Agent is a datapacket ant
			// Log trace info
			if (trace_)
				trace_->sample(this, O_Ants2006_DATAPACKETCOUNT );
		} else {
			// Log trace info
			if (trace_)
				trace_->sample(this, O_Ants2006_FORWARDCOUNT);
		}
		
		// Set color in nam
		ip->flowid() = 0;

	case SWS_FORWARDSEARCH:
		
		// Store copy of packet. To be used during local boost.
		if( swarmagent_->behaviour() == O_Ants2006_normal && local_boost_) {
			if (last_forward_agentpacket_) 
				Packet::free(last_forward_agentpacket_);
			last_forward_agentpacket_ = p_->copy();
		}
		// Update rate estimate of forward ants (normal + explorer)
		if (swarmagent_->behaviour() == O_Ants2006_normal || swarmagent_->behaviour() == O_Ants2006_explorer) {
			double interarrival_time = now() - last_forward_time_;
			forward_interarrival_time_samples_.push_back(interarrival_time);
			double old_sample = 0;
			if (forward_interarrival_time_samples_.size() >= swarmagent_->alfa() ) {
				old_sample = forward_interarrival_time_samples_.front();
				forward_interarrival_time_samples_.pop_front();
			}
			forward_interarrival_sum_ += interarrival_time - old_sample;
			forward_rate_estimate_ = ( forward_interarrival_sum_ > 0 ? forward_interarrival_time_samples_.size() /  forward_interarrival_sum_ : 0);
			//forward_rate_estimate_ = forward_rate_estimate_ * swarmagent_->alfa() + (1.0 - swarmagent_->alfa()) / (now() - last_forward_time_);

			//			if (node_address() == 4 )
			//printf("%e rate %e (%e, %e)\n", now(), forward_rate_estimate_,  swarmagent_->alfa(), last_forward_time_);

			last_forward_time_ = now();
		}
			

		
		if (node_address() != ip->daddr() || swarmagent_->tabulist().empty()) {
			// Not yet at destination...

		
			// Build list of neighbour nodes not already visited.
			vector<pair<NsObject*, double> > new_neighbours;
			vector<pair<Neighbour*, double> > neigh_new_neighbours; //(Oriol)
			Neighbour* neigh_homenode_entry; //(Oriol)
			int j; //(Oriol)xapussa
			double pherolevel_sum = 0;
			double pherolevel_sum2 = 0; //(Oriol) SUM of pheromones2
			bool homenode_is_neighbour = false;
			NsObject* homenode_entry = NULL;
			for (list<Neighbour*>::iterator i = neighbourhood_.begin();  i != neighbourhood_.end(); i++) {

				// Check for changes in neighbourhood
				if (last_neighbourhood_.find(*i) != last_neighbourhood_.end() &&
					last_neighbourhood_[*i] != (! (*i)->linkdown()) ) {
					// Change in link state for old neighbour.
					if (local_boost_)
						// Execute local boosting
						local_boost((*i)->entry());
				}
				// Remember neighbour and link state
				last_neighbourhood_[*i] = (! (*i)->linkdown());
					
				// Check if link to neighbour is up/down
				if ( (*i)->linkdown() )
					// Skip neighbour
					continue;

				// Check if neighbour has been visited
				bool neighbour_visited = false;
				//(Oriol) Initial behavior, storing IDs in the agent
				if(STORE_PORTS_BEHAVIOR==0)
				{
					for (list<pair<nsaddr_t, NsObject*> >::iterator j = swarmagent_->tabulist().begin();
						 j != swarmagent_->tabulist().end(); j++) {
						if ( (*i)->addr()  ==  j->first ) {
							neighbour_visited = true;
						    break;
						}
					}
				}
				if(STORE_PORTS_BEHAVIOR==1)//(Oriol) behavior storing Ports in the agent
				{
					for (list<pair<nsaddr_t, NsObject*> >::iterator j = swarmagent_->tabulist().begin();
						 j != swarmagent_->tabulist().end(); j++) {
						 list<pair<nsaddr_t, NsObject*> >::iterator prova=j;
							prova++;
						if ( ((*i)->addr()  ==  j->first) && ((prova)==swarmagent_->tabulist().end())) {
							neighbour_visited = true;
						    break;
						}
					}					
					
					
				}

				if (! neighbour_visited) {
					// Unvisited (new) neighbour found
					
					if (TWO_PHERO_BEHAVIOR==0) //(Oriol) use of 1 pheromone behavior
					{
						// Find pheromone level (if any) towards neigbour
						PheroTableKey pk((*i)->entry(), swarmagent_->pheroid());
						PheroTable::iterator p = pherotable_.find(pk);
						pherolevel_sum += (p != pherotable_.end() ? (p->second).pherolevel() : 0);

						// Store link entry towards neigbour and accumulated phero level towards neigbour
						new_neighbours.push_back(pair<NsObject*, double>((*i)->entry(), pherolevel_sum));
						neigh_new_neighbours.push_back(pair<Neighbour*, double>(*i, 0));//(Oriol)
					}

					if (TWO_PHERO_BEHAVIOR==1)//(Oriol) use of 2 pheromones behavior
					{
						// Find pheromone level (if any) towards neigbour
						if(/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)//(Oriol) flag that means which behavior have a particular agent, 0=> first pheromone :: 1=> second pheromone
						{
							
							PheroTableKey pk((*i)->entry(), swarmagent_->pheroid());
							PheroTable::iterator p = pherotable_.find(pk);
							pherolevel_sum += (p != pherotable_.end() ? (p->second).pherolevel() : 0);

							// Store link entry towards neigbour and accumulated phero level towards neigbour
							new_neighbours.push_back(pair<NsObject*, double>((*i)->entry(), pherolevel_sum));
							neigh_new_neighbours.push_back(pair<Neighbour*, double>(*i, 0));//(Oriol)

						}else{
							
							PheroTableKey pk((*i)->entry(), swarmagent_->pheroid());
							PheroTable::iterator p = pherotable2_.find(pk);
							pherolevel_sum2 += (p != pherotable2_.end() ? (p->second).pherolevel() : 0);
						
							// Store link entry towards neigbour and accumulated phero level towards neigbour
							new_neighbours.push_back(pair<NsObject*, double>((*i)->entry(), pherolevel_sum2));
							neigh_new_neighbours.push_back(pair<Neighbour*, double>(*i, 0));//(Oriol)
						}
					}



				}
				// Check if home node is neighbour
				if ( ! homenode_is_neighbour && (*i)->addr() == ip->daddr()) {
					homenode_is_neighbour = true;
					homenode_entry = (*i)->entry();
					neigh_homenode_entry = *i;//(Oriol)
				}
			}

			// (Oriol) for the agents always can revisit all nodes in store ports behavior
			if (STORE_PORTS_BEHAVIOR==0)
			{
				if (new_neighbours.empty()) {
					// All neighbours have been visited
					if(homenode_is_neighbour) {
						// Return to homenode
						new_neighbours.push_back(pair<NsObject*, double>(homenode_entry, 1));
						neigh_new_neighbours.push_back(pair<Neighbour*, double>(neigh_homenode_entry, 0));//(Oriol)
						pherolevel_sum = 1;
						if (TWO_PHERO_BEHAVIOR==1) pherolevel_sum2=1; //(Oriol)
					} else {
						// Allow revisit to any neighbour
						for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++) {
							new_neighbours.push_back(pair<NsObject*, double>((*i)->entry(), 0));
							neigh_new_neighbours.push_back(pair<Neighbour*, double>(*i, 0));//(Oriol)
						}
					}
				}
			}	 
			if (STORE_PORTS_BEHAVIOR==1)//(Oriol)
			{
				if (new_neighbours.empty()) {
					for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++) {
						new_neighbours.push_back(pair<NsObject*, double>((*i)->entry(), 0));
						neigh_new_neighbours.push_back(pair<Neighbour*, double>(*i, 0));//(Oriol)
					}
				}
			}
			
			// Select which neighbour to visit next by applying pheromone distribution
			NsObject* nextnode = NULL;
			Neighbour* neigh_nextnode = NULL; //(Oriol) to save the next node in neighbour format
			if (TWO_PHERO_BEHAVIOR==0) //(Oriol)
			{
				if (pherolevel_sum == 0 || swarmagent_->behaviour() == O_Ants2006_explorer ) { 
					// No pheromones found or explorer behaviour set. Make a uniform distribution.
					long n = randomgen_.rand_int(1, new_neighbours.size());
					nextnode = new_neighbours[n-1].first;
					neigh_nextnode= neigh_new_neighbours[n-1].first;//(Oriol)
				} else {
					// Apply "roulett wheel" method base on pheromone distribution to select next node
					double u = randomgen_.uniform_double();
					j=0;//(Oriol)
					for (vector<pair<NsObject*, double> >::iterator i = new_neighbours.begin(); i != new_neighbours.end(); i++) {
						//printf("<%e %d>, ", i->second, ((entry2neighbour_.find(i->first))->second)->addr());
						
						if ( u <= (i->second / pherolevel_sum)) {
							// Select node to be next
							nextnode = i->first;
							neigh_nextnode= neigh_new_neighbours[j].first;//(Oriol)
							
							break;
						}
						j++;
					}
					//printf("\n");
				}
			}else{//(Oriol)
				if ((pherolevel_sum == 0 && /*swarmagent_->go_second()==0*/ swarmagent_->from1st()==0) || (pherolevel_sum2 == 0 && /*swarmagent_->go_second()==1 &&*/ swarmagent_->from1st()==1) || swarmagent_->behaviour() == O_Ants2006_explorer ) { 
					// No pheromones found or explorer behaviour set. Make a uniform distribution.
					long n = randomgen_.rand_int(1, new_neighbours.size());
					nextnode = new_neighbours[n-1].first;
					neigh_nextnode= neigh_new_neighbours[n-1].first;//(Oriol)
				} else {
					// Apply "roulett wheel" method base on pheromone distribution to select next node
					double u = randomgen_.uniform_double();
					j=0;
					for (vector<pair<NsObject*, double> >::iterator i = new_neighbours.begin(); i != new_neighbours.end(); i++) {
						//printf("<%e %d>, ", i->second, ((entry2neighbour_.find(i->first))->second)->addr());
						if(/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)
						{
							if ( u <= (i->second / pherolevel_sum)) {
								// Select node to be next
								nextnode = i->first;
								neigh_nextnode=neigh_new_neighbours[j].first;//(Oriol)
								break;
							}
						}else{
							if ( u <= (i->second / pherolevel_sum2)) {
								// Select node to be next
								nextnode = i->first;
								neigh_nextnode= neigh_new_neighbours[j].first;//(Oriol)
								break;
							}
						}
						j++;

					}
					//printf("\n");
				}
			}

			//Update packetsize
			//ch->size() = swarmagent_->size() + sizeof(hdr_cmn) + sizeof(hdr_swarm);
			//ch->size() *= 10; // Scale up a little to make packet visible in nam
			//ch->size() = 1;
			if(swarmagent_->behaviour()!=O_Ants2006_datapacket)
			{
				if (STORE_PORTS_BEHAVIOR==0) ch->size() = ch->size() + 8*6; //(Oriol) Each node stored is 6 bytes MAC adress
				else ch->size() = ch->size() + 8; // (Oriol) Maximum switches of 256 ports
			}else{
				ch->size()=1520;
			}
			// Record current visit and entry to next node
			
			swarmagent_->tabulist().push_back(pair<nsaddr_t, NsObject*>(node_address(), nextnode));

			if (trace_phero_ && (trace_pheros_.empty() or trace_pheros_.find(swarmagent_->pheroid()) != trace_pheros_.end() ) ) 
				// Sample pheromone values
				trace_->sample(this, O_Ants2006_PHEROMONES ); 
			
			//(Oriol) Touch to trace second pheromone (this and all the features)
		

			if (TWO_PHERO_BEHAVIOR==1) //(Oriol)
			{
				if (trace_phero2_ && (trace_pheros2_.empty() or trace_pheros2_.find(swarmagent_->pheroid()) != trace_pheros2_.end() ) ) 
					// Sample pheromone values
					trace_->sample(this, O_Ants2006_PHEROMONES2 ); 
			}


				//Here, the cost function has to be defined (Oriol)


				//(Oriol) cost functions of Tdelay
                double ocupation;

				ocupation= neigh_nextnode->traffic() / neigh_nextnode->bandwidth();
				printf("occupation=%lf \n", ocupation);
				if((/*ocupation <MAX_OCUPATION &&*/ (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0) && TWO_PHERO_BEHAVIOR==1) || TWO_PHERO_BEHAVIOR==0)
				{ 
					if(TWO_MEASURE_BEHAVIOR==0)
					{
						//el 100000000 es perque capiga en un double sense quedar-se en 0
						swarmagent_->L_path() = swarmagent_->L_path() + 100000000*1000*((ch->size()/neigh_nextnode->bandwidth())*(1/(1-ocupation)) + neigh_nextnode->retard());
						if (swarmagent_->L_path() ==0) swarmagent_->L_path()=0.000001;
						if (ocupation > MAX_OCUPATION && TWO_PHERO_BEHAVIOR==1) 
						{
							swarmagent_->go_second()=1;
							swarmagent_->go_first()=1;
						}
					
					}else{
						if(ocupation<MAX_OCUPATION)
						{
							//This cost is the transfer time exactly, but using a mean of ocupation
							swarmagent_->L_path() = swarmagent_->L_path() + 100000000*1000*((ch->size()/neigh_nextnode->bandwidth())*(1/(1-ocupation)) + neigh_nextnode->retard());
							if (swarmagent_->L_path() ==0) swarmagent_->L_path()=0.000001;
							printf("Cost with cost function 1 is %lf, with cost function 2 would be %lf\n", 100000000*1000*((ch->size()/neigh_nextnode->bandwidth())*(1/(1-ocupation)) + neigh_nextnode->retard()), 100000000*1000*((ch->size()+neigh_nextnode->bitsQueueDrT())/neigh_nextnode->bandwidth() + neigh_nextnode->retard()));
						}else{
							//This is the real transfer time for a link
							printf("Cost with cost function 1 would be %lf, with cost function 2 is %lf\n", 100000000*1000*((ch->size()/neigh_nextnode->bandwidth())*(1/(1-ocupation)) + neigh_nextnode->retard()), 100000000*1000*((ch->size()+neigh_nextnode->bitsQueueDrT())/neigh_nextnode->bandwidth() + neigh_nextnode->retard()));
							//printf("llarg cua=%lf\n",neigh_nextnode->bitsQueueDrT());
							swarmagent_->L_path()=swarmagent_->L_path() + 100000000*1000*((ch->size() + neigh_nextnode->bitsQueueDrT())/neigh_nextnode->bandwidth() + neigh_nextnode->retard()); 
							if (swarmagent_->L_path() ==0) swarmagent_->L_path()=0.000001;
						}
					}
				}else{

					
					swarmagent_->L_path2() = swarmagent_->L_path2() + 100000000*1000*((ch->size() + neigh_nextnode->bitsQueueDrT())/neigh_nextnode->bandwidth() + neigh_nextnode->retard());
					if (swarmagent_->L_path2() ==0) swarmagent_->L_path2()=0.000001;
					if(ocupation > MAX_OCUPATION) 
					{
						swarmagent_->go_first()=1;
						swarmagent_->go_second()=1;
					}
				}

				//cost_actual_=swarmagent_->L_path();

				//cost_actual2_=swarmagent_->L_path2();
				//printf("Despres del calcul de cost: %f\n", cost_actual_);

			// Move to next node (i.e. send agent state to next node)
			nextnode->recv(p_,h_);
			// Trigger local boost if appropriate
			//local_boost();

			break;


		} else {
			// Agent is at destination, prepare for backtracking
			
			// Record current visit 
			swarmagent_->tabulist().push_back(pair<nsaddr_t, NsObject*>(node_address(), NULL));

			if (swarmagent_->behaviour() == O_Ants2006_datapacket) {
				// Agent is a datapacket ant, is simulation a data-packet
				// Log trace info
				if (trace_)
				{
					trace_->sample(this, O_Ants2006_PATH |  O_Ants2006_DATAPACKETRECEIVED | O_Ants2006_DATAPACKETCOST);
					if(TWO_PHERO_BEHAVIOR==1) trace_->sample(this, O_Ants2006_DATAPACKETCOST2); //(Oriol)
				}
				//printf("Despres de tracar datapacketcost %f\n",cost_actual_);
				// Terminate agent
				Packet::free(p_);
				break;
			}
			//printf("Quan l'agent es a la destinacio\n");	
			if (TWO_PHERO_BEHAVIOR==0)
			{
				if (gammaautoreg_.find(swarmagent_->pheroid()) == gammaautoreg_.end()) {
					// First agent with this pheroid. Init misc variables
					rho_reduction_factor_[swarmagent_->pheroid()] = 1;
					best_cost_[swarmagent_->pheroid()] = MAXDOUBLE;          
					period_since_new_best_[swarmagent_->pheroid()] = 0;
				}
			}else{
				if (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)
				{
					if (gammaautoreg_.find(swarmagent_->pheroid()) == gammaautoreg_.end()) {
						// First agent with this pheroid. Init misc variables
						rho_reduction_factor_[swarmagent_->pheroid()] = 1;
						best_cost_[swarmagent_->pheroid()] = MAXDOUBLE;          
						period_since_new_best_[swarmagent_->pheroid()] = 0;

					}
				}else{
					if (gammaautoreg2_.find(swarmagent_->pheroid()) == gammaautoreg2_.end()) {
						// First agent with this pheroid. Init misc variables
						rho_reduction_factor_[swarmagent_->pheroid()] = 1;
						best_cost2_[swarmagent_->pheroid()] = MAXDOUBLE;          
						period_since_new_best2_[swarmagent_->pheroid()] = 0;
					}
				}
			}
			//printf("Despres de tocar la gamma quan l'agent es a la destinacio\n");		

			// Calculate cost
			//double L_path = now() - swarmagent_->birthtime();
			//(Oriol) Calculate cost due to swarmagent_->L_path() that has been recorded during all the path
			double L_path;
			if(TWO_PHERO_BEHAVIOR==0 || (TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)))
			{
				L_path=swarmagent_->L_path();
			}else{
				L_path=swarmagent_->L_path2();
			}
			
			//if(now() > 268.0 && now() < 269.0 )
			//	printf("%f L_path: %f Bh: %d\n", now(), L_path,  swarmagent_->behaviour());

			// Perform rho-reduction DISABLED
			//			rho_reduction(L_path);
			
			swarmagent_->rho() *= rho_reduction_factor_[swarmagent_->pheroid()];
			
			// Calculate new temperatur (gamma) for all agents //(Oriol) the if
			if (TWO_PHERO_BEHAVIOR==0 || (TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0))) recalc_gamma(gammaautoreg_[swarmagent_->pheroid()], swarmagent_->L_path(), swarmagent_->beta(), swarmagent_->rho());

			//(Oriol)
			if (TWO_PHERO_BEHAVIOR==1 && /*swarmagent_->go_second()==1 &&*/ swarmagent_->from1st()==1) recalc_gamma(gammaautoreg2_[swarmagent_->pheroid()], swarmagent_->L_path2(), swarmagent_->beta(), swarmagent_->rho());
			//printf("Abans de tracar la gamma, L_path=%lf\n", L_path);
			// Log trace info
			if (trace_)
			{
				//printf("Despres d'entrar a l'if de tracar la gamma\n");
				if (TWO_PHERO_BEHAVIOR==0) trace_->sample(this, O_Ants2006_GAMMA_ALL | O_Ants2006_ELITELIMIT | O_Ants2006_PATH | O_Ants2006_COST_ALL | O_Ants2006_ARRIVEDCOUNT);
				if (TWO_PHERO_BEHAVIOR==1) trace_->sample(this, O_Ants2006_GAMMA_ALL | O_Ants2006_ELITELIMIT | O_Ants2006_PATH | O_Ants2006_COST_ALL | O_Ants2006_ARRIVEDCOUNT | O_Ants2006_GAMMA_ALL2 | O_Ants2006_COST_ALL2);
				//printf("Abans de sortir de lif de la gamma");
			}
			//printf("Despres de tracar la gamma, L_path2=%lf\n", swarmagent_->L_path2());
			// Check if path found is in "elite set"(Oriol)
			if(TWO_PHERO_BEHAVIOR==0 || TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 || */ swarmagent_->from1st()==0))
			{
				if (  swarmagent_->behaviour() == O_Ants2006_normal &&
					  swarmagent_->L_path() >= (- log(swarmagent_->rho()) * gammaautoreg_[swarmagent_->pheroid()].gamma())) {
					// Path not in elite set
					// Terminate agent
					Packet::free(p_);
					break;
				}
			}else{
				if (  swarmagent_->behaviour() == O_Ants2006_normal &&
					  swarmagent_->L_path2() >= (- log(swarmagent_->rho()) * gammaautoreg2_[swarmagent_->pheroid()].gamma())) {
					// Path not in elite set
					// Terminate agent
					Packet::free(p_);
					break;
				}
			}


			// Calculate new temperatur (gamma) for elite agents (Oriol)
			if (TWO_PHERO_BEHAVIOR==0 || (TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)))
			{
				recalc_gamma(gammaautoreg_elite_[swarmagent_->pheroid()], L_path, swarmagent_->beta(), swarmagent_->rho());
				// Store gamma and path cost as agent state
				swarmagent_->gamma() = gammaautoreg_elite_[swarmagent_->pheroid()].gamma();
				//swarmagent_->L_path() = L_path;
				swarmagent_->gamma2() = gammaautoreg_elite2_[swarmagent_->pheroid()].gamma();
			}
			if (TWO_PHERO_BEHAVIOR==1 && /* swarmagent_->go_second()==1 &&*/ swarmagent_->from1st()==1)
			{
				recalc_gamma(gammaautoreg_elite2_[swarmagent_->pheroid()], L_path, swarmagent_->beta(), swarmagent_->rho());
				// Store gamma and path cost as agent state
				swarmagent_->gamma() = gammaautoreg_elite_[swarmagent_->pheroid()].gamma();
				swarmagent_->gamma2() = gammaautoreg_elite2_[swarmagent_->pheroid()].gamma();
				//swarmagent_->L_path2() = L_path;
			}
			//printf("Despres de recalcular la gamma");

			// Set color in nam
			ip->flowid() = 1;
			// Reset ttl
			ip->ttl() = swarmagent_->max_ttl();

			// Log trace info
			if (trace_)
			{
				if (TWO_PHERO_BEHAVIOR==0) trace_->sample(this, O_Ants2006_COST | O_Ants2006_GAMMA_ELITE | O_Ants2006_GAMMA_DIFF | O_Ants2006_GAMMA_REL | O_Ants2006_RHO | O_Ants2006_BACKTRACKCOUNT);
				if (TWO_PHERO_BEHAVIOR==1) trace_->sample(this, O_Ants2006_COST | O_Ants2006_GAMMA_ELITE | O_Ants2006_GAMMA_DIFF | O_Ants2006_GAMMA_REL | O_Ants2006_RHO | O_Ants2006_BACKTRACKCOUNT | O_Ants2006_COST2 | O_Ants2006_GAMMA_ELITE2 | O_Ants2006_GAMMA_DIFF2 | O_Ants2006_GAMMA_REL2);
			}
			//printf("Abans de passar backtracking a state");
			swarmagent_->state() = SWS_BACKTRACKING;
			// (... continous into next case ...)
		}

	case SWS_BACKTRACKING: {
		
		// Get entry to previously visited node.
		NsObject* previous_node = 	swarmagent_->tabulist().back().second;
		
		//		printf("DEBUG bt %f %ld %d<-", now(), (long)swarmagent_, node_address());
		//printf("Comenca el backtracking");
		if(previous_node) {
			
			//printf ("%d ", ((entry2neighbour_.find(swarmagent_->tabulist().back().second))->second)->addr() );
		
			// Update pheromones (Oriol)
			double L_on_gamma;
			if (TWO_PHERO_BEHAVIOR==0 || (TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)))
			{
				if(swarmagent_->gamma()!=0) L_on_gamma = swarmagent_->L_path() / swarmagent_->gamma();
				else L_on_gamma = swarmagent_->L_path() / 0.000001;
			}else{
				if(swarmagent_->gamma2()!=0) L_on_gamma = swarmagent_->L_path2() / swarmagent_->gamma2();
				else L_on_gamma = swarmagent_->L_path2() / 0.000001;
			}
			double H = exp( - L_on_gamma );
			for (list<Neighbour*>::iterator i = neighbourhood_.begin();  i != neighbourhood_.end(); i++) {

				// Check if neighbour is node visited last time
				bool neigbour_is_last = ((*i)->entry() == previous_node);
			
				// Update pheromones (Oriol)
				PheroAutoreg& p = pherotable_[PheroTableKey((*i)->entry(), swarmagent_->pheroid())];
				if (TWO_PHERO_BEHAVIOR==0 || (TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)))
				{
					p = pherotable_[PheroTableKey((*i)->entry(), swarmagent_->pheroid())];
					p.pherolevel() = (neigbour_is_last? H : 0) +
						p.A() + 
						( p.C() * p.B() == 0 || swarmagent_->gamma() > (2 * p.C() / p.B()) ? 
					 	 - p.B() / swarmagent_->gamma() + p.C() / (swarmagent_->gamma() * swarmagent_->gamma()) : 
					 	 - p.B() * p.B() / (4 * p.C() ) );
				}else{
					p = pherotable2_[PheroTableKey((*i)->entry(), swarmagent_->pheroid())];

						p.pherolevel() = (neigbour_is_last? H : 0) +
							p.A() + 
							( p.C() * p.B() == 0 || swarmagent_->gamma2() > (2 * p.C() / p.B()) ? 
						 	 - p.B() / swarmagent_->gamma2() + p.C() / (swarmagent_->gamma2() * swarmagent_->gamma2()) : 
						 	 - p.B() * p.B() / (4 * p.C() ) );
					
				}
				
				
				
				
				// Check for "underflow"
				if (p.pherolevel() < 0) {
					// Reset all phero parameters
					p.pherolevel() = 0;
					p.A() = 0;
					p.B() = 0;
					p.C() = 0;
				} else {
					// Update pheromone autoreg parameters
					p.A() = swarmagent_->beta() * (p.A() + (neigbour_is_last ? 
															H*(1+L_on_gamma * (1+L_on_gamma/2)) : 0) );
					if (TWO_PHERO_BEHAVIOR==0 || (TWO_PHERO_BEHAVIOR==1 && (/*swarmagent_->go_second()==0 ||*/ swarmagent_->from1st()==0)))
					{
						p.B() = swarmagent_->beta() * (p.B() + (neigbour_is_last ? 
																H*(swarmagent_->L_path() + swarmagent_->L_path() * 
																   swarmagent_->L_path() / swarmagent_->gamma() ) : 0) );
						p.C() = swarmagent_->beta() * (p.C() + (neigbour_is_last ? 
															H*(swarmagent_->L_path() * swarmagent_->L_path() / 2 ) : 0) );
					}else{
						p.B() = swarmagent_->beta() * (p.B() + (neigbour_is_last ? 
																H*(swarmagent_->L_path2() + swarmagent_->L_path2() * 
																   swarmagent_->L_path2() / swarmagent_->gamma2() ) : 0) );
						p.C() = swarmagent_->beta() * (p.C() + (neigbour_is_last ? 
																H*(swarmagent_->L_path2() * swarmagent_->L_path2() / 2 ) : 0) );
					}
					
				}
			}
		}
		
		//		printf("\n");
		

		// Update rate estimate of backtracking (elite) ants
		double interarrival_time = now() - last_backtrack_time_;
		backtrack_interarrival_time_samples_.push_back(interarrival_time);
		double old_sample = 0;
		if (backtrack_interarrival_time_samples_.size() >= swarmagent_->alfa() ) {
			old_sample = backtrack_interarrival_time_samples_.front();
			backtrack_interarrival_time_samples_.pop_front();
		}
		backtrack_interarrival_sum_ += interarrival_time - old_sample;
		backtrack_rate_estimate_ = (backtrack_interarrival_sum_ > 0 ? backtrack_interarrival_time_samples_.size() / backtrack_interarrival_sum_ : 0);
		//backtrack_rate_estimate_ = backtrack_rate_estimate_ * swarmagent_->alfa() + (1.0 - swarmagent_->alfa()) / (now() - last_backtrack_time_);
		last_backtrack_time_ = now();
				
		// Remove current node from tabulist
		swarmagent_->tabulist().pop_back();
		if (swarmagent_->tabulist().empty()) {
			// Backtracking completed
			//(Oriol) This is to check if there are a lot of paths with queue
			if(TWO_PHERO_BEHAVIOR==1)
			{
				if (swarmagent_->go_first()==0) swarmagent_->go_second()=0; //(Oriol) in order to change from queues to not queues
				if( swarmagent_->go_second()==1)
				{
					
					ind_queues_=BETA_QUEUES*ind_queues_ + (1-BETA_QUEUES)*1;
				}else{
					ind_queues_=BETA_QUEUES*ind_queues_;
				}
			}
			//printf("\nAcaba el backtracking\n");
			// Terminate agent
			Packet::free(p_);
			break;
		}
		
		
		// Move to next node
		Neighbour* next_node = (addr2neighbour_.find( swarmagent_->tabulist().back().first ) )->second;
		if (next_node )
			(next_node->entry())->recv(p_, h_);
		else 
			// Link between nodes in tabulist no longer exist
			Packet::free(p_);
		break;
	}
	default:
		printf("O_Ants2006_SwarmRtM::runSwarmAgent: Invalide swarm agent state %d\n", swarmagent_->state());
		Packet::free(p_);
		break;
	}

	// Process next queued packet (if any)
	poll_queue();

}
void O_Ants2006_SwarmRtM::rho_reduction(double L_path)
	// Adjust rho if when convergence is considered too slow
{
	if(! swarmagent_) 
		//Do nothing
		return;
	/*
	if (swarmagent_->behaviour() == O_Ants2006_normal && 
		gammaautoreg_.gamma() - gammaautoreg_elite_.gamma() > swarmagent_->gammadiff_limit()) {
		// Difference between  gamma for all agents and elite agents is exceeding limit
	*/
	if (best_cost_[swarmagent_->pheroid()] > L_path) { 
		// New best cost
		best_cost_[swarmagent_->pheroid()] = L_path;
		period_since_new_best_[swarmagent_->pheroid()] = 0;
	} else
		period_since_new_best_[swarmagent_->pheroid()] += 1;
	
	if (period_since_new_best_[swarmagent_->pheroid()] > swarmagent_->D() ) {
		// Adjust rho reduction factor
		rho_reduction_factor_[swarmagent_->pheroid()] *= 0.95;
		period_since_new_best_[swarmagent_->pheroid()] = 0;

		// printf("%f Adjusting rho for %d: %le\n ", now(), swarmagent_->pheroid(),  rho_reduction_factor_[swarmagent_->pheroid()]); fflush(NULL);
		
	}

}

void O_Ants2006_SwarmRtM::recalc_gamma(GammaAutoreg& gammaauto, double L_path, double beta, double rho)
	// Calculate new temperatur (gamma) and update autoreg parameters for gamma
{
			if (gammaauto.A() == 0 &&
				gammaauto.B() == 0 &&
				gammaauto.gamma() == 0 &&
				gammaauto.t() == 0)
				// Init. gamma
				gammaauto.gamma() = - L_path / log(rho);
			
			double beta2t = 0;
			// A large t is approxed with t=infinity.
			if (gammaauto.t() < T_INFINITY_APPROX) {
				gammaauto.t() += 1;
				 beta2t = pow(beta, gammaauto.t());
			}
			double L_on_gamma;
			if(gammaauto.gamma()!=0) L_on_gamma = L_path / gammaauto.gamma();
			else  L_on_gamma = L_path / 0.000001;
			double H = exp( - L_on_gamma );

			double new_gamma =  (gammaauto.B() + L_path * H) / 
				(gammaauto.A() + H * (1+ L_on_gamma) - rho * 
				 ((1 - beta2t)/(1 - beta)));

			// Check for "underflow"
			if (new_gamma > 0 ) 
				gammaauto.gamma() = new_gamma;
			else
				printf("O_Ants2006_SwarmRtM::recalc_gamma: Gamma less than 0 (%lf).\n", new_gamma);

			// Update autoreg parameters for gamma
			if(gammaauto.gamma()!=0) L_on_gamma = L_path / gammaauto.gamma();
			else L_on_gamma = L_path / 0.000001;
			H = exp( - L_on_gamma );

			gammaauto.A() = beta * (gammaauto.A() + H * (1 + L_on_gamma));
			gammaauto.B() = beta * (gammaauto.B() + L_path * H);

}




float O_Ants2006_SwarmRtM::get_cost()
	// Return cost recoded by current swarm agent
{
	
	return swarmagent_->L_path();
	
}

float O_Ants2006_SwarmRtM::get_cost2()
	// Return cost recoded by current swarm agent
{
	return swarmagent_->L_path2();
}

float O_Ants2006_SwarmRtM::get_gamma()
	// Return gamma (temperature) recoded by current swarm agent
	// This gamma is controlled by elite ants
{
	return swarmagent_->gamma();
}
//(Oriol):
float O_Ants2006_SwarmRtM::get_gamma2()
	// Return gamma (temperature) recoded by current swarm agent
	// This gamma is controlled by elite ants
{
	return swarmagent_->gamma2();
}

float O_Ants2006_SwarmRtM::get_gamma_all()
	// Return gamma (temperature) updated by all ants
{
	return gammaautoreg_[swarmagent_->pheroid()].gamma();
}
//(Oriol):
float O_Ants2006_SwarmRtM::get_gamma_all2()
	// Return gamma (temperature) updated by all ants
{
	return gammaautoreg2_[swarmagent_->pheroid()].gamma();
}

list<pair<nsaddr_t,NsObject*> >&  O_Ants2006_SwarmRtM::get_path()  
	// Return path traversed by current swarm agent
{
	return swarmagent_->tabulist();
}

float O_Ants2006_SwarmRtM::get_delay()
	// Return delay experienced by current swarm agent
{
	return 0;
}

float O_Ants2006_SwarmRtM::get_elitelimit()
	// Return limit for being in elite set
{
	return (- log(swarmagent_->rho()) * gammaautoreg_[swarmagent_->pheroid()].gamma());
}
//(Oriol):
float O_Ants2006_SwarmRtM::get_elitelimit2()
	// Return limit for being in elite set
{
	return (- log(swarmagent_->rho()) * gammaautoreg2_[swarmagent_->pheroid()].gamma());
}

float O_Ants2006_SwarmRtM::get_rho()
	// Return rho applied
{
	return swarmagent_->rho();
}

int O_Ants2006_SwarmRtM::get_pheroid()
	// Return pheromone id
{
	return swarmagent_->pheroid();
}

int O_Ants2006_SwarmRtM::get_behaviour()
		// Return behaviour flag (0 = normal, 1 = explorer)
{
	return swarmagent_->behaviour();
}

map<nsaddr_t, double>&  O_Ants2006_SwarmRtM::get_phero()  
	// Return current pheromone value for current agent
{
	phero_list_.clear();
		
	for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++) {
		// Find pheromone level (if any) for current agent towards neigbour
		PheroTableKey pk((*i)->entry(), swarmagent_->pheroid());
		PheroTable::iterator p = pherotable_.find(pk);
		phero_list_[((entry2neighbour_.find((*i)->entry()))->second)->addr()] = (p != pherotable_.end() ? (p->second).pherolevel() : 0);
	}

	return phero_list_;
}

map<nsaddr_t, double>&  O_Ants2006_SwarmRtM::get_phero2()  //(Oriol)
	// Return current pheromone value for current agent
{
	phero_list2_.clear();
		
	for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++) {
		// Find pheromone level (if any) for current agent towards neigbour
		PheroTableKey pk((*i)->entry(), swarmagent_->pheroid());
		PheroTable::iterator p = pherotable2_.find(pk);
		phero_list2_[((entry2neighbour_.find((*i)->entry()))->second)->addr()] = (p != pherotable2_.end() ? (p->second).pherolevel() : 0);
	}

	return phero_list_;
}


double O_Ants2006_SwarmRtM::get_backtrack_rate()
		// Return estimated rate of backtracking agents
{
	return backtrack_rate_estimate_;
}

void O_Ants2006_SwarmRtM::local_boost(NsObject* failed_link_entry)
	// Execute local boost if required, i.e. generate a set of additional forward agents
{
	// Prepare prob. dist. (pdf) based on "pheromones"
	vector<pair<NsObject*, double> > neighbours;
	double pherolevel_sum = 0;
	for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++) {
		
		// Check if link to neighbour is up/down
		if ( (*i)->linkdown() )
			// Skip neighbour
			continue;

		// Find pheromone level (if any) towards neigbour
		PheroTableKey pk((*i)->entry(), swarmagent_->pheroid());
		PheroTable::iterator p = pherotable_.find(pk);
		double phero_level =  (p != pherotable_.end() ? (p->second).pherolevel() : 0);
		pherolevel_sum +=  phero_level;
		
		// Store link entry towards neigbour and accumulated phero level towards neigbour
		neighbours.push_back(pair<NsObject*, double>((*i)->entry(), phero_level));
	}

	// Get pheromone level on failed link too
	PheroTableKey pk(failed_link_entry, swarmagent_->pheroid());
	PheroTable::iterator p = pherotable_.find(pk);
	double phero_level_failed_link =  (p != pherotable_.end() ? (p->second).pherolevel() : 0);
	
	// Calculate size of boost (equ. 9 in paper, and rate weighted by pheromonlevel on failed link)
	double current_rate =  ((O_Ants2006_data*)last_forward_agentpacket_->userdata())->current_rate();
	double rate_weight = phero_level_failed_link / (pherolevel_sum + phero_level_failed_link);
	double boost_size = min_replica_ * 
		( 1 + (neighbourhood_.size() - 1 ) * (forward_rate_estimate_ * rate_weight / current_rate));

	printf("%f local boost at %d:\n\tForward rate estimate %f\n\tCurrent rate %f\n\tRate weight %f\n\tBoost size %f\n\tBoost distr.: ", 
		   now(), node_address(), forward_rate_estimate_, current_rate, rate_weight, boost_size);
	
	for (vector<pair<NsObject*, double> >::iterator i = neighbours.begin(); i != neighbours.end(); i++) {
		// Generate a boost of agents relative to pheromone level towards neighbour
		int num_agents = (int)( boost_size * (i->second / pherolevel_sum) );
		
		// Generate boosts of agents. Apply uniform distribution.
		//int num_agents = (int)(boost_size / neighbours.size() );
		
		num_agents = (num_agents > 0 ? num_agents : 1 );     // Round up 

		printf("%d ->%d, ", num_agents, ((entry2neighbour_.find(i->first))->second)->addr());
		
		for (int j=0; j < num_agents; j++) {
			
			// Prepare copy of last forward agent sent
			Packet* p = last_forward_agentpacket_->copy(); 

			// Send packet
			(i->first)->recv(p,h_);
			
		}
	}
	printf("\n"); fflush(NULL);
	
}
