// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/O_Ants2006/O_Ants2006_swarmrtm.h,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: O_Ants2006_ii_impl_rate swarm agent system
//       Holds code for both swarm node and swarm agent functionality.
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-11-08


#ifndef ns_O_Ants2006_swarmrtm_h
#define ns_O_Ants2006_swarmrtm_h

#include <list>
#include <set>
#include "delay.h"
#include "swarm-rtmodule.h"
#include "O_Ants2006_trace.h"
#include "CEants.h"

#define SERVICE_TIME  0.001       // Simulated swarm agent processing time

// Ant behaviour types
#define O_Ants2006_normal 0
#define O_Ants2006_explorer 1
#define O_Ants2006_datapacket 2

class O_Ants2006_data : public AppData {
	// Packet data holding state of swarm agent
public:
	O_Ants2006_data(int behaviour, int pheroid, int max_ttl, 
				  double beta, double rho, double D, double alfa, double current_rate) : AppData(SWARM_DATA) {
		// Init.
		state_ = SWS_IDLE;
		behaviour_ = behaviour;
		pheroid_ = pheroid;
		max_ttl_ = max_ttl;

		alfa_ = alfa;
		current_rate_ = current_rate;

		tabulist_ = new list<pair<nsaddr_t, NsObject*> >();

		birthtime_ = 0;
		
		gamma_ = 0;
		gamma2_ = 0; //(Oriol)
		beta_ = beta;
		rho_ = rho;
		D_ = D;
		L_path_ = 0;
		L_path2_ = 0;//(Oriol)
	}

	O_Ants2006_data(O_Ants2006_data& d) : AppData(d) {
		// Make this a copy of "d"
		state_ = d.state_;
		behaviour_ = d.behaviour_;
		pheroid_ = d.pheroid_;
		max_ttl_ = d.max_ttl_;

		alfa_ = d.alfa_;
		current_rate_ = d.current_rate_;

		tabulist_ = new list<pair<nsaddr_t, NsObject*> >(d.tabulist());
		//tabulist_ = new list<pair<nsaddr_t, NsObject*> >();

		birthtime_ = d.birthtime_;

		gamma_ = d.gamma_;
		gamma2_ = d.gamma2_;//(Oriol)
		beta_ = d.beta_;
		rho_ = d.rho_;
		D_ = d.D_;
		L_path_ = d.L_path_;
		L_path2_= d.L_path2_;//(Oriol)
	}

	virtual ~O_Ants2006_data() {
		delete tabulist_;
		//delete tabuentrylist_;
	}

	// Access methods
	inline int& state() {return state_; };
	inline int& behaviour() {return behaviour_;};
	inline int& pheroid() {return pheroid_; };
	inline int& max_ttl() {return max_ttl_; };
	inline int& go_second() {return go_second_; };//(Oriol)
	inline int& go_first() {return go_first_; };//(Oriol)
	inline int& from1st() {return from1st_; };//(Oriol)
	inline list<pair<nsaddr_t, NsObject*> >& tabulist() {return *tabulist_; };
	//	inline list<NsObject*>& tabuentrylist() {return *tabuentrylist_; };
	//inline NsObject*& last_node() {return last_node_;};
	inline float& birthtime() {return birthtime_; };
	inline double& gamma() {return gamma_;};
	inline double& gamma2() {return gamma2_;};
	inline double& beta() {return beta_;};
	inline double& rho() {return rho_;};
	inline double& D() {return D_;};
	inline double& L_path() {return L_path_;};
	inline double& L_path2() { return L_path2_;};
	inline double& alfa() {return alfa_;};
	inline double& current_rate() {return current_rate_;};

	virtual int size() const { return 1; //sizeof(state_) + 
//								   sizeof(pheroid_) + 
	//							   tabulist_->size() * sizeof(nsaddr_t); }
}

	virtual AppData* copy() { return new O_Ants2006_data(*this); }

protected:
	// Swarm agent state info
	int state_;                  // State of agent
	int behaviour_;               // Behaviour type
	int pheroid_;                // Swarm agent species id
	int max_ttl_;                // Initial value for ttl field
	int go_second_;				// (Oriol) Flag that says if continue with first or second pheromone (0 and 1)
	int go_first_;				//(Oriol)
	int from1st_;				//(Oriol)

	double alfa_;                // Weight for geometric rate averaging
	double current_rate_;        // Agent generation rate when this agent was created

	list<pair<nsaddr_t, NsObject*> >* tabulist_;         // List of already visited nodes
	//	list<NsObject*>* tabuentrylist_;   // List of entries towards already visited nodes
	//NsObject* last_node_;                          // Pointer to entry of last node

	float birthtime_;            // Time of "birth"

	double gamma_;               // "Temperature"
	double gamma2_;			 // "Temperature" (Oriol)
	double beta_;                // "Evaporation factor"
	double rho_;                 // Search focus
	double D_;                   // Applied for rho reduction
	double L_path_;              // Path cost
	double L_path2_;			 // Path cost 2 (Oriol)
};


class O_Ants2006_SwarmRtM : public SwarmRtModule  {
	// Swarm system routing module 
	// implementing O_Ants2006_ii_impl_rate ant
public:
	O_Ants2006_SwarmRtM();

	virtual const char* module_name() const { return "O_Ants2006"; }
	virtual void expire(Event *);                           // Timeout handler
	virtual int command(int argc, char const *const *argv);
	
	float get_cost();                  //(Oriol) Return cost recoded by current swarm agent
	float get_cost2();                  // (Oriol)Return cost recoded by current swarm agent
	float get_gamma();                 // Return gamma (temperatur) recoded by current swarm agent (elite)
	float get_gamma2();                // (Oriol) Return gamma (temperatur) recoded by current swarm agent (elite) 2
	float get_gamma_all();             // Return gamma (temperatur) for all agents
	float get_gamma_all2();            // (Oriol) Return gamma (temperatur) for all agents 2
	list<pair<nsaddr_t,NsObject*> >& get_path();        // Return path traversed by current swarm agent
	float get_delay();                 // Return delay experienced by current swarm agent
	float get_elitelimit();            // Return limit for elite set membership
	float get_elitelimit2();           // (Oriol) Return limit for elite set membership 2
	float get_rho();                   // Return rho applied

	inline double& ind_queues() {return ind_queues_;};	//(Oriol)
	int get_pheroid();                 // Return pheromone id
	int get_behaviour();               // Return behaviour flag (0 = normal, 1 = explorer)
	map<nsaddr_t, double>&  get_phero();  // Return map of pheromone values for current agent
	map<nsaddr_t, double>&  get_phero2(); //(Oriol) Return map of pheromone values for current agent
	double get_backtrack_rate();          // Return estimated rate of backtracking agents

protected:
	void recalc_gamma(GammaAutoreg&, double, double, double);	// Calculate new temperatur (gamma)
	void rho_reduction(double);           // Adjust rho if no new best path has been found by the last D agents

	O_Ants2006_data* swarmagent_;           // Pointer to current visiting swarm agent data structure    

	double ultim_temps_;
	float cost_actual_;					 //(Oriol) Actual cost
	float cost_actual2_;					 //(Oriol) Actual cost2
	PheroTable pherotable_;                  // Table of pheromones towards neighbour nodes
	PheroTable pherotable2_;                 // (Oriol) Table of pheromones towards neighbour nodes
	double ind_queues_;					 //(Oriol) flag to know if agent has to be sent with go_second set on.
	bool trace_phero_;                       // Flag for enabling trace of pheromone value
	bool trace_phero2_;                      // (Oriol) Flag for enabling trace of pheromone value
	set<int> trace_pheros_   ;               // Set of phero ids to be traced
	set<int> trace_pheros2_   ;              // (Oriol) Set of phero ids to be traced
	map<nsaddr_t,double> phero_list_;        // Map of neighbour-node-address and pherolevels
	map<nsaddr_t,double> phero_list2_;       // (Oriol) Map of neighbour-node-address and pherolevels

	// (Maps below are indexed by pheroid)
	map<int,GammaAutoreg> gammaautoreg_;           // Temperature autoregression parameters 
	map<int,GammaAutoreg> gammaautoreg_elite_;     // Temperature autoregression parameters for elite O_Ants2006_ii_impl_rates
	map<int,double> best_cost_;                    // Best cost found so far
	map<int,double> period_since_new_best_;        // Time period since a new best cost was found
	map<int,double> rho_reduction_factor_;         // Factor applied to tighten rho
	map<int,GammaAutoreg> gammaautoreg2_;           // (Oriol) Temperature autoregression parameters 
	map<int,GammaAutoreg> gammaautoreg_elite2_;     // (Oriol) Temperature autoregression parameters for elite O_Ants2006_ii_impl_rates
	map<int,double> best_cost2_;                    // (Oriol) Best cost found so far
	map<int,double> period_since_new_best2_;        // (Oriol) Time period since a new best cost was found


	double backtrack_rate_estimate_;                   // Etimate of rate of backtracking ants
	double last_backtrack_time_;              // Arrival time of latest backtracking ant
	list<double> backtrack_interarrival_time_samples_;  // Window of interarrival time samples
	double backtrack_interarrival_sum_;                // Sum of samples in window

	double forward_rate_estimate_;            // Etimates of rate of incoming forward ants
	double last_forward_time_;                // Arrival time of latest forward ant
	Packet* last_forward_agentpacket_;        // Copy of last forward agent packet	
	list<double> forward_interarrival_time_samples_;  // Window of interarrival time samples
	double forward_interarrival_sum_;                // Sum of samples in window

	bool local_boost_;                        // Flag indicating if local boosting/generation of agents should happend on link failures
	int min_replica_;                         // Minimum no of forward ant copies generated as the result of a local boost
	map<Neighbour*, bool> last_neighbourhood_;       // State of connections to neighbour nodes the last time a packet/agent arrived
	void local_boost(NsObject*);                       // Execute local boost
	map<int,double> current_normal_rate_;         // Current rate reported for normal agents
	map<int,double> current_explorer_rate_;       // Current rate reported for explorer agents

	
};

#endif
