// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/O_Ants2006/O_Ants2006_trace.cc,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: Trace handler for O_Ants2006 swarm agent system. Handles multiple pheromone ids
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-11-07

#include <list>
#include <algorithm>

#include "O_Ants2006_trace.h"
#include "O_Ants2006_swarmrtm.h"
#include "O_Ants2006_generator.h"


#define MAX_POWER  500       // Limit of y when x^y and x = <0,1>
                             // before x^y ~= 0 

static class O_Ants2006_TraceClass : public TclClass {
public:
  O_Ants2006_TraceClass() : TclClass("SwarmTrace/O_Ants2006") {}
  TclObject* create(int argc, const char*const* argv) {
	  if (argc == 6)
		  return (new O_Ants2006_Trace(argv[4], argv[5]));
	  else 
		  return (new O_Ants2006_Trace("",""));
  	  
  }
} class_O_Ants2006;

O_Ants2006_Trace::O_Ants2006_Trace(const char* binsize, const char* samplefreq)
{
	prefix_ = "O_Ants2006";

	float bsize = 0;   
	if (sscanf(binsize, "%f", &bsize) != 1)
		printf("O_Ants2006_Trace::O_Ants2006_Trace: Bin size not set.\n");
	// Set geometric weight to make sample no. "binsize" 
	// back in time weighted by 0.3
	geo_weight_ = bsize > 0 ? exp(log(0.3)/bsize) : 0 ;

	// Set sample frequency
	float freq = 0;
	if (sscanf(samplefreq, "%f", &freq) != 1)
		printf("O_Ants2006_Trace::O_Ants2006_Trace: Sampling frequency not set.\n");
	sampleperiod_ = (freq > 0 ? 1/freq : 0);

	// Init. inst. vars

}

O_Ants2006_Trace::~O_Ants2006_Trace()
{
}


void O_Ants2006_Trace::sample(SwarmRtModule* swarmrtm, int flags)
  // Sampletrace values according to flags
{
  if (flags & O_Ants2006_COST) log_cost((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_PATH) log_path((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_DELAY) log_delay((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_GAMMA_ELITE) log_gamma_elite((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_GAMMA_ALL) log_gamma_all((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_GAMMA_DIFF) log_gamma_diff((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_GAMMA_REL) log_gamma_rel((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_ELITELIMIT) log_elitelimit((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_BACKTRACKCOUNT) log_backtrackcount((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_FORWARDCOUNT) log_forwardcount((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_RHO) log_rho((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_PHEROMONES) log_phero((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_DATAPACKETCOUNT) log_datapacketcount((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_DATAPACKETRECEIVED) log_datapacketreceived((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_DATAPACKETCOST) log_datapacketcost((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_COST_ALL) log_cost_all((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_ARRIVEDCOUNT) log_arrivedcount((O_Ants2006_SwarmRtM*)swarmrtm);
  if (flags & O_Ants2006_COST2) log_cost2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_GAMMA_ELITE2) log_gamma_elite2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_GAMMA_ALL2) log_gamma_all2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_GAMMA_DIFF2) log_gamma_diff2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_GAMMA_REL2) log_gamma_rel2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_DATAPACKETCOST2) log_datapacketcost2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_COST_ALL2) log_cost_all2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  if (flags & O_Ants2006_PHEROMONES2) log_phero2((O_Ants2006_SwarmRtM*)swarmrtm);//(Oriol)
  fflush(NULL);
}


void O_Ants2006_Trace::sample(Agent* agent, int flags)
  // Sampletrace values according to flags
{
  if (flags & O_Ants2006_GENRATE) log_genrate((O_Ants2006_generator*)agent);
}

void O_Ants2006_Trace::log_cost(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current cost from swarm routing module
{
	int pheroid = swarmrtm->get_pheroid();
	if (cost_last_sample_.find(pheroid) == cost_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		cost_avg_[pheroid] = 0;
		cost_2nd_moment_[pheroid] = 0;
		cost_count_[pheroid] = 0;
		cost_log_[pheroid] = NULL;
		cost_avg_log_[pheroid] = NULL;
		cost_var_log_[pheroid] = NULL;
		cost_varcoeff_log_[pheroid] = NULL;
		cost_last_sample_[pheroid] = 0;
	}

	if ( now() - cost_last_sample_[pheroid] >= sampleperiod_ ) {
		// Update geometrically weighted average
		if (cost_count_[pheroid] < MAX_POWER) {
			cost_count_[pheroid]++;
			double weight_both = 1/(1 - pow(geo_weight_,cost_count_[pheroid]));
			double weight_old = (1- pow(geo_weight_,cost_count_[pheroid] - 1 )) * geo_weight_;
			double weight_new = (1-geo_weight_);

			/*
			cost_avg_ = (1/(1 - pow(geo_weight_,cost_count_[pheroid]))) * (
				(1- pow(geo_weight_,cost_count_[pheroid] - 1 )) * geo_weight_ * cost_avg_[pheroid] + 
				(1-geo_weight_) * swarmrtm->get_cost() 
				);
			*/
			cost_avg_[pheroid] = weight_both * (  weight_old * cost_avg_[pheroid] + weight_new * swarmrtm->get_cost() );
			cost_2nd_moment_[pheroid] = weight_both * (  weight_old * cost_2nd_moment_[pheroid] + 
												weight_new * swarmrtm->get_cost() * swarmrtm->get_cost() );

		} else {
			// Assume cost_count_ = infinity
			cost_avg_[pheroid] = geo_weight_ * cost_avg_[pheroid] +  (1-geo_weight_) * swarmrtm->get_cost();
			cost_2nd_moment_[pheroid] = geo_weight_ * cost_2nd_moment_[pheroid] +  (1-geo_weight_) * swarmrtm->get_cost() * swarmrtm->get_cost();
		}			

		// Log sample
		log_printf(cost_log_[pheroid], pheroid, "cost", "%f %f\n", now(), swarmrtm->get_cost());
		log_printf(cost_avg_log_[pheroid], pheroid, "costavg", "%f %f\n", now(), cost_avg_[pheroid]);
		double cost_var =  cost_2nd_moment_[pheroid] - cost_avg_[pheroid] * cost_avg_[pheroid];
		log_printf(cost_var_log_[pheroid], pheroid, "costvar", "%f %f\n", now(), cost_var);
		if (cost_avg_[pheroid]!=0) log_printf(cost_varcoeff_log_[pheroid], pheroid, "costvarcoeff", "%f %f\n", now(),  cost_var / cost_avg_[pheroid]);
		else log_printf(cost_varcoeff_log_[pheroid], pheroid, "costvarcoeff", "%f %f\n", now(),  cost_var / 0.0000001);
		cost_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_cost2(O_Ants2006_SwarmRtM* swarmrtm) //(Oriol)
  // Fetch and log current cost from swarm routing module
{
	int pheroid = swarmrtm->get_pheroid();
	if (cost_last_sample2_.find(pheroid) == cost_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		cost_avg2_[pheroid] = 0;
		cost_2nd_moment2_[pheroid] = 0;
		cost_count2_[pheroid] = 0;
		cost_log2_[pheroid] = NULL;
		cost_avg_log2_[pheroid] = NULL;
		cost_var_log2_[pheroid] = NULL;
		cost_varcoeff_log2_[pheroid] = NULL;
		cost_last_sample2_[pheroid] = 0;
	}

	if ( now() - cost_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Update geometrically weighted average
		if (cost_count2_[pheroid] < MAX_POWER) {
			cost_count2_[pheroid]++;
			double weight_both = 1/(1 - pow(geo_weight_,cost_count2_[pheroid]));
			double weight_old = (1- pow(geo_weight_,cost_count2_[pheroid] - 1 )) * geo_weight_;
			double weight_new = (1-geo_weight_);

			/*
			cost_avg_ = (1/(1 - pow(geo_weight_,cost_count_[pheroid]))) * (
				(1- pow(geo_weight_,cost_count_[pheroid] - 1 )) * geo_weight_ * cost_avg_[pheroid] + 
				(1-geo_weight_) * swarmrtm->get_cost() 
				);
			*/
			cost_avg2_[pheroid] = weight_both * (  weight_old * cost_avg2_[pheroid] + weight_new * swarmrtm->get_cost2() );
			cost_2nd_moment2_[pheroid] = weight_both * (  weight_old * cost_2nd_moment2_[pheroid] + 
												weight_new * swarmrtm->get_cost2() * swarmrtm->get_cost2() );

		} else {
			// Assume cost_count_ = infinity
			cost_avg2_[pheroid] = geo_weight_ * cost_avg2_[pheroid] +  (1-geo_weight_) * swarmrtm->get_cost2();
			cost_2nd_moment2_[pheroid] = geo_weight_ * cost_2nd_moment2_[pheroid] +  (1-geo_weight_) * swarmrtm->get_cost2() * swarmrtm->get_cost2();
		}			

		// Log sample
		log_printf(cost_log2_[pheroid], pheroid, "cost2", "%f %f\n", now(), swarmrtm->get_cost2());
		log_printf(cost_avg_log2_[pheroid], pheroid, "costavg2", "%f %f\n", now(), cost_avg2_[pheroid]);
		double cost_var =  cost_2nd_moment2_[pheroid] - cost_avg2_[pheroid] * cost_avg2_[pheroid];
		log_printf(cost_var_log2_[pheroid], pheroid, "costvar2", "%f %f\n", now(), cost_var);
		if(cost_avg2_[pheroid]!=0) log_printf(cost_varcoeff_log2_[pheroid], pheroid, "costvarcoeff2", "%f %f\n", now(),  cost_var / cost_avg2_[pheroid]);
		else log_printf(cost_varcoeff_log2_[pheroid], pheroid, "costvarcoeff2", "%f %f\n", now(),  cost_var / 0.0000001);
		cost_last_sample2_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_path(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current path from swarm routing module
{
	//printf("He entrat a tracar path\n");
	int pheroid = swarmrtm->get_pheroid();
	if (path_last_sample_.find(pheroid) == path_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		path_log_[pheroid] = NULL;
		path_last_sample_[pheroid] = 0;
	}

	if ( now() - path_last_sample_[pheroid] >= sampleperiod_ ) {
		// Create string version of path
		list<pair<nsaddr_t, NsObject*> >& path = swarmrtm->get_path();
		char* pathstr = new char[path.size() * 11];          // Handles 10 digit node addresses 
		*pathstr = 0;
		char* path_idx = pathstr; 
		
		for (list<pair<nsaddr_t, NsObject*> >::iterator i=path.begin(); 
			 i != path.end();  i++) {
			sprintf(path_idx,"%d ", i->first);
			path_idx += strlen(path_idx);
		}
		
		// Log sample
		log_printf(path_log_[pheroid], pheroid, "path", "%f [%s] %f %d\n", now(), pathstr,  swarmrtm->get_cost(), swarmrtm->get_behaviour());
		delete pathstr;
		
		path_last_sample_[pheroid] = now();
	}
		//printf("He sortit de tracar path\n");
}

void O_Ants2006_Trace::log_delay(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current delay from swarm routing module
{
  
}

void O_Ants2006_Trace::log_gamma_elite(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current gamma elite from swarm routing module
{
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_elite_last_sample_.find(pheroid) == gamma_elite_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_elite_log_[pheroid] = NULL;
		gamma_elite_last_sample_[pheroid] = 0;
	}

	if ( now() - gamma_elite_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(gamma_elite_log_[pheroid], pheroid, "gamma_elite", "%f %f\n", now(), swarmrtm->get_gamma());

		gamma_elite_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_gamma_elite2(O_Ants2006_SwarmRtM* swarmrtm)//(Oriol)
  // Fetch and log current gamma elite from swarm routing module
{
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_elite_last_sample2_.find(pheroid) == gamma_elite_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_elite_log2_[pheroid] = NULL;
		gamma_elite_last_sample2_[pheroid] = 0;
	}

	if ( now() - gamma_elite_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(gamma_elite_log2_[pheroid], pheroid, "gamma_elite2", "%f %f\n", now(), swarmrtm->get_gamma2());

		gamma_elite_last_sample2_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_gamma_all(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current gamma form all agents from swarm routing module
{
	//printf("He entrat a tracar gamma_all\n");
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_all_last_sample_.find(pheroid) == gamma_all_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_all_log_[pheroid] = NULL;
		gamma_all_last_sample_[pheroid] = 0;
	}

	if ( now() - gamma_all_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(gamma_all_log_[pheroid], pheroid, "gamma_all", "%f %f\n", now(), swarmrtm->get_gamma_all());

		gamma_all_last_sample_[pheroid] = now();
	}
	//printf("He sortit de tracar gamma_all\n");
}

void O_Ants2006_Trace::log_gamma_all2(O_Ants2006_SwarmRtM* swarmrtm)//(Oriol)
  // Fetch and log current gamma form all agents from swarm routing module
{
	//printf("He entrat a tracar gamma_all2\n");
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_all_last_sample2_.find(pheroid) == gamma_all_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_all_log2_[pheroid] = NULL;
		gamma_all_last_sample2_[pheroid] = 0;
	}

	if ( now() - gamma_all_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(gamma_all_log2_[pheroid], pheroid, "gamma_all2", "%f %f\n", now(), swarmrtm->get_gamma_all2());

		gamma_all_last_sample2_[pheroid] = now();
	}
	//printf("He sortit de tracar gamma_all2\n");
}

void O_Ants2006_Trace::log_gamma_diff(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log differens between current gamma form all agents and elite agents
{
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_diff_last_sample_.find(pheroid) == gamma_diff_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_diff_log_[pheroid] = NULL;
		gamma_diff_last_sample_[pheroid] = 0;
	}

	if ( now() - gamma_diff_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(gamma_diff_log_[pheroid], pheroid, "gamma_diff", "%f %f\n", now(), swarmrtm->get_gamma_all() - swarmrtm->get_gamma());

		gamma_diff_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_gamma_diff2(O_Ants2006_SwarmRtM* swarmrtm)//(Oriol)
  // Fetch and log differens between current gamma form all agents and elite agents
{
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_diff_last_sample2_.find(pheroid) == gamma_diff_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_diff_log2_[pheroid] = NULL;
		gamma_diff_last_sample2_[pheroid] = 0;
	}

	if ( now() - gamma_diff_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(gamma_diff_log2_[pheroid], pheroid, "gamma_diff2", "%f %f\n", now(), swarmrtm->get_gamma_all2() - swarmrtm->get_gamma2());

		gamma_diff_last_sample2_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_gamma_rel(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log relation between current gamma form all agents and elite agents
{
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_rel_last_sample_.find(pheroid) == gamma_rel_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_rel_log_[pheroid] = NULL;
		gamma_rel_last_sample_[pheroid] = 0;
	}

	if ( now() - gamma_rel_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		if (swarmrtm->get_gamma_all() != 0)
			log_printf(gamma_rel_log_[pheroid], pheroid, "gamma_rel", "%f %f\n", now(), swarmrtm->get_gamma()/swarmrtm->get_gamma_all());

		gamma_rel_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_gamma_rel2(O_Ants2006_SwarmRtM* swarmrtm)//(Oriol)
  // Fetch and log relation between current gamma form all agents and elite agents
{
	int pheroid = swarmrtm->get_pheroid();
	if (gamma_rel_last_sample2_.find(pheroid) == gamma_rel_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		gamma_rel_log2_[pheroid] = NULL;
		gamma_rel_last_sample2_[pheroid] = 0;
	}

	if ( now() - gamma_rel_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Log sample
		if (swarmrtm->get_gamma_all2() != 0)
			log_printf(gamma_rel_log2_[pheroid], pheroid, "gamma_rel2", "%f %f\n", now(), swarmrtm->get_gamma2()/swarmrtm->get_gamma_all2());

		gamma_rel_last_sample2_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_elitelimit(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log limit for elite set membership from swarm routing module
{
		//printf("He entrat a tracar elitelimit\n");
	int pheroid = swarmrtm->get_pheroid();
	if ( elitelimit_last_sample_.find(pheroid) == elitelimit_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		elitelimit_log_[pheroid] = NULL;
		elitelimit_last_sample_[pheroid] = 0;
	}

	if ( now() - elitelimit_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(elitelimit_log_[pheroid], pheroid, "elitelimit", "%f %f\n", now(), swarmrtm->get_elitelimit());

		elitelimit_last_sample_[pheroid] = now();
	}
		//printf("He sortit de tracar elitelimit\n");
}

void O_Ants2006_Trace::log_backtrackcount(O_Ants2006_SwarmRtM* swarmrtm)
  // Count and log no of backtrack agents
{
	int pheroid = swarmrtm->get_pheroid();
	if ( backtrackcount_last_sample_.find(pheroid) == backtrackcount_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		backtrackcount_log_[pheroid] = NULL;
		backtrackcount_last_sample_[pheroid] = 0;
		backtrackcount_[pheroid] = 0;
	}

	backtrackcount_[pheroid]++;
	if ( now() - backtrackcount_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(backtrackcount_log_[pheroid], pheroid, "backtrackcount", "%f %ld\n", now(), backtrackcount_[pheroid]);

		backtrackcount_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_forwardcount(O_Ants2006_SwarmRtM* swarmrtm)
  // Count and log no of forward agents
{

	int pheroid = swarmrtm->get_pheroid();
	if (forwardcount_last_sample_.find(pheroid) == forwardcount_last_sample_.end() ) {
		// First time a sample is logged For this pheromone id. Init.
		forwardcount_log_[pheroid] = NULL;
		forwardcount_last_sample_[pheroid] = 0;
		forwardcount_[pheroid] = 0;
	}

	forwardcount_[pheroid]++;
	if ( now() - forwardcount_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(forwardcount_log_[pheroid], pheroid, "forwardcount", "%f %ld\n", now(), forwardcount_[pheroid]);

		forwardcount_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_rho(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log rho applied
{
	int pheroid = swarmrtm->get_pheroid();
	if (rho_last_sample_.find(pheroid) == rho_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		rho_log_[pheroid] = NULL;
		rho_last_sample_[pheroid] = 0;
	}

	if ( now() - rho_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(rho_log_[pheroid], pheroid, "rho", "%f %f\n", now(), swarmrtm->get_rho());

		rho_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_phero(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log pheromone values
{
	int pheroid = swarmrtm->get_pheroid();
	nsaddr_t from_node = swarmrtm->node_address();
	if (phero_last_sample_.find(from_node) == phero_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		phero_log_[from_node] = NULL;
		phero_last_sample_[from_node] = 0;
		phero_last_level_[from_node].clear();
	}

	map<nsaddr_t, double>& phero = swarmrtm->get_phero();
	// Update last phero sample DB
	for(map<nsaddr_t, double>::iterator i = phero.begin(); i != phero.end(); i++){
		nsaddr_t to_node = i->first;
		((phero_last_level_[from_node])[to_node])[pheroid] = i->second;
	}
	
	if ( now() - phero_last_sample_[from_node] >= sampleperiod_ ) {
		// Log last samples for current node (from_node)
		for(map<nsaddr_t,map<int,double> >::iterator i = phero_last_level_[from_node].begin(); 
			i !=  phero_last_level_[from_node].end(); i++){
			// Log time and to-node
			nsaddr_t to_node = i->first;
			log_printf(phero_log_[from_node], from_node, "phero", "%f %d ", now(), to_node);

			// Log phero level for all phero ids
			for(map<int,double>::iterator j = i->second.begin(); j != i->second.end(); j++)
				log_printf(phero_log_[from_node], from_node, "phero", "%f ", j->second);

			// (Add newline)
			log_printf(phero_log_[from_node], from_node, "phero", "\n");
				
		}

		phero_last_sample_[from_node] = now();
	}
}

void O_Ants2006_Trace::log_phero2(O_Ants2006_SwarmRtM* swarmrtm) //(Oriol)
  // Fetch and log pheromone values
{
	int pheroid = swarmrtm->get_pheroid();
	nsaddr_t from_node = swarmrtm->node_address();
	if (phero_last_sample2_.find(from_node) == phero_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		phero_log2_[from_node] = NULL;
		phero_last_sample2_[from_node] = 0;
		phero_last_level2_[from_node].clear();
	}

	map<nsaddr_t, double>& phero = swarmrtm->get_phero2();
	// Update last phero sample DB
	for(map<nsaddr_t, double>::iterator i = phero.begin(); i != phero.end(); i++){
		nsaddr_t to_node = i->first;
		((phero_last_level2_[from_node])[to_node])[pheroid] = i->second;
	}
	
	if ( now() - phero_last_sample2_[from_node] >= sampleperiod_ ) {
		// Log last samples for current node (from_node)
		for(map<nsaddr_t,map<int,double> >::iterator i = phero_last_level2_[from_node].begin(); 
			i !=  phero_last_level2_[from_node].end(); i++){
			// Log time and to-node
			nsaddr_t to_node = i->first;
			log_printf(phero_log2_[from_node], from_node, "phero2", "%f %d ", now(), to_node);

			// Log phero level for all phero ids
			for(map<int,double>::iterator j = i->second.begin(); j != i->second.end(); j++)
				log_printf(phero_log2_[from_node], from_node, "phero2", "%f ", j->second);

			// (Add newline)
			log_printf(phero_log2_[from_node], from_node, "phero2", "\n");
				
		}

		phero_last_sample2_[from_node] = now();
	}
}

void O_Ants2006_Trace::log_datapacketcount(O_Ants2006_SwarmRtM* swarmrtm)
  // Count and log no of datapacket agents
{

	int pheroid = swarmrtm->get_pheroid();
	if (datapacketcount_last_sample_.find(pheroid) == datapacketcount_last_sample_.end() ) {
		// First time a sample is logged For this pheromone id. Init.
		datapacketcount_log_[pheroid] = NULL;
		datapacketcount_last_sample_[pheroid] = 0;
		datapacketcount_[pheroid] = 0;
	}

	datapacketcount_[pheroid]++;
	if ( now() - datapacketcount_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(datapacketcount_log_[pheroid], pheroid, "datapacketcount", "%f %ld\n", now(), datapacketcount_[pheroid]);

		datapacketcount_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_datapacketreceived(O_Ants2006_SwarmRtM* swarmrtm)
  // Count and log no of datapacket agents arrived at destination
{

	int pheroid = swarmrtm->get_pheroid();
	if (datapacketreceived_last_sample_.find(pheroid) == datapacketreceived_last_sample_.end() ) {
		// First time a sample is logged For this pheromone id. Init.
		datapacketreceived_log_[pheroid] = NULL;
		datapacketreceived_last_sample_[pheroid] = 0;
		datapacketreceived_[pheroid] = 0;
	}

	datapacketreceived_[pheroid]++;
	if ( now() - datapacketreceived_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(datapacketreceived_log_[pheroid], pheroid, "datapacketreceived", "%f %ld\n", now(), datapacketreceived_[pheroid]);

		datapacketreceived_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_genrate(O_Ants2006_generator* agent)
  // Log current rate of a generator
{

	if (genrate_last_sample_.find(agent) == genrate_last_sample_.end() ) {
		// First time a sample is logged For this pheromone id. Init.
		genrate_log_[agent] = NULL;
		genrate_last_sample_[agent] = 0;
	}

	if ( now() - genrate_last_sample_[agent] >= sampleperiod_ ) {
		// Log sample
		log_printf(genrate_log_[agent], agent->addr(), "genrate", "%f %f\n", now(), agent->get_rate());

		genrate_last_sample_[agent] = now();
	}
}

void O_Ants2006_Trace::log_datapacketcost(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current datapacketcost from swarm routing module
{
	int pheroid = swarmrtm->get_pheroid();
	if (datapacketcost_last_sample_.find(pheroid) == datapacketcost_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		datapacketcost_log_[pheroid] = NULL;
		datapacketcost_last_sample_[pheroid] = 0;
	}

	if ( now() - datapacketcost_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample

		log_printf(datapacketcost_log_[pheroid], pheroid, "datapacketcost", "%f %f\n", now(), swarmrtm->get_cost());
		datapacketcost_last_sample_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_datapacketcost2(O_Ants2006_SwarmRtM* swarmrtm)//(Oriol)
  // Fetch and log current datapacketcost from swarm routing module
{
	int pheroid = swarmrtm->get_pheroid();
	if (datapacketcost_last_sample2_.find(pheroid) == datapacketcost_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		datapacketcost_log2_[pheroid] = NULL;
		datapacketcost_last_sample2_[pheroid] = 0;
	}

	if ( now() - datapacketcost_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(datapacketcost_log2_[pheroid], pheroid, "datapacketcost2", "%f %f\n", now(), swarmrtm->get_cost2());

		datapacketcost_last_sample2_[pheroid] = now();
	}
}

void O_Ants2006_Trace::log_cost_all(O_Ants2006_SwarmRtM* swarmrtm)
  // Fetch and log current cost_all from swarm routing module
{
	//printf("He entrat a tracar cost_all\n");
	int pheroid = swarmrtm->get_pheroid();
	if (cost_all_last_sample_.find(pheroid) == cost_all_last_sample_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		cost_all_log_[pheroid] = NULL;
		cost_all_last_sample_[pheroid] = 0;
	}

	if ( now() - cost_all_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample

		log_printf(cost_all_log_[pheroid], pheroid, "cost_all", "%f %f\n", now(), swarmrtm->get_cost());

		cost_all_last_sample_[pheroid] = now();
	}
	//printf("He sortit tracar cost_all\n");
}

void O_Ants2006_Trace::log_cost_all2(O_Ants2006_SwarmRtM* swarmrtm)//(Oriol)
  // Fetch and log current cost_all from swarm routing module
{
	//printf("He entrat a tracar cost_all2\n");
	int pheroid = swarmrtm->get_pheroid();
	if (cost_all_last_sample2_.find(pheroid) == cost_all_last_sample2_.end() ) {
		// First time a sample is logged for this pheromone id. Init.
		cost_all_log2_[pheroid] = NULL;
		cost_all_last_sample2_[pheroid] = 0;
	}

	if ( now() - cost_all_last_sample2_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(cost_all_log2_[pheroid], pheroid, "cost_all2", "%f %f\n", now(), swarmrtm->get_cost2());

		cost_all_last_sample2_[pheroid] = now();
	}
	//printf("He sortit de tracar cost_all2\n");
}

void O_Ants2006_Trace::log_arrivedcount(O_Ants2006_SwarmRtM* swarmrtm)
  // Count and log no of agents arrived at destination
{
		//printf("He entrat a tracar arrivedcount\n");
	int pheroid = swarmrtm->get_pheroid();
	if (arrivedcount_last_sample_.find(pheroid) == arrivedcount_last_sample_.end() ) {
		// First time a sample is logged For this pheromone id. Init.
		arrivedcount_log_[pheroid] = NULL;
		arrivedcount_last_sample_[pheroid] = 0;
		arrivedcount_[pheroid] = 0;
	}

	arrivedcount_[pheroid]++;
	if ( now() - arrivedcount_last_sample_[pheroid] >= sampleperiod_ ) {
		// Log sample
		log_printf(arrivedcount_log_[pheroid], pheroid, "arrivedcount", "%f %ld\n", now(), arrivedcount_[pheroid]);

		arrivedcount_last_sample_[pheroid] = now();
	}
			//printf("He sortit de tracar arrivedcount\n");
}
