// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/O_Ants2006/O_Ants2006_trace.h,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: Trace handler for O_Ants2006_ii_impl_rate swarm agent system. Handles multiple pheronom ids.
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-11-07

#ifndef ns_O_Ants2006_trace_h
#define ns_O_Ants2006_trace_h

#include <map>

#include "swarm-trace.h"
#include "agent.h"

// Trace sampling flags //(Oriol)
#define O_Ants2006_COST               0x0000001
#define O_Ants2006_PATH               0x0000002
#define O_Ants2006_DELAY              0x0000004
#define O_Ants2006_GAMMA_ELITE        0x0000008
#define O_Ants2006_GAMMA_ALL          0x0000010
#define O_Ants2006_GAMMA_DIFF         0x0000020
#define O_Ants2006_GAMMA_REL          0x0000040
#define O_Ants2006_ELITELIMIT         0x0000080
#define O_Ants2006_RHO                0x0000100
#define O_Ants2006_BACKTRACKCOUNT     0x0000200
#define O_Ants2006_FORWARDCOUNT       0x0000400
#define O_Ants2006_PHEROMONES         0x0000800
#define O_Ants2006_DATAPACKETCOUNT    0x0001000
#define O_Ants2006_DATAPACKETRECEIVED 0x0002000
#define O_Ants2006_DATAPACKETCOST     0x0004000
#define O_Ants2006_GENRATE            0x0008000
#define O_Ants2006_COST_ALL           0x0010000
#define O_Ants2006_ARRIVEDCOUNT       0x0020000
#define O_Ants2006_COST2			  0x0040000
#define O_Ants2006_GAMMA_ELITE2       0x0080000
#define O_Ants2006_GAMMA_ALL2         0x0100000
#define O_Ants2006_GAMMA_DIFF2        0x0200000
#define O_Ants2006_GAMMA_REL2         0x0400000
#define O_Ants2006_DATAPACKETCOST2    0x0800000
#define O_Ants2006_COST_ALL2		  0x1000000
#define O_Ants2006_PHEROMONES2   	  0x2000000


class O_Ants2006_SwarmRtM;   // (To ensure class def is in scope)
class O_Ants2006_generator;   // (To ensure class def is in scope)

class O_Ants2006_Trace : public SwarmTrace {
public:
	O_Ants2006_Trace(const char*, const char*);
	~O_Ants2006_Trace();
  
	// Sample trace data
	void sample(SwarmRtModule*, int);
	void sample(Agent*, int);

protected:
	
	float geo_weight_;         // Weight for geometriclly weighted average
	float sampleperiod_;       // Min period between samples to be logged

	map<int,float> cost_avg_;            // Log value accumulator
	map<int,float> cost_2nd_moment_;            // Log value accumulator
	map<int,int>   cost_count_;          // Log value counter
	map<int,FILE*> cost_log_;            // Log file handle
	map<int,FILE*> cost_avg_log_;         // Log file handle
	map<int,FILE*> cost_var_log_;         // Log file handle
	map<int,FILE*> cost_varcoeff_log_;         // Log file handle
	map<int,float> cost_last_sample_;    // Time when last sample was logged
	void log_cost(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,float> cost_avg2_;            // Log value accumulator
	map<int,float> cost_2nd_moment2_;            // Log value accumulator
	map<int,int>   cost_count2_;          // Log value counter
	map<int,FILE*> cost_log2_;            // Log file handle
	map<int,FILE*> cost_avg_log2_;         // Log file handle
	map<int,FILE*> cost_var_log2_;         // Log file handle
	map<int,FILE*> cost_varcoeff_log2_;         // Log file handle
	map<int,float> cost_last_sample2_;    // Time when last sample was logged
	void log_cost2(O_Ants2006_SwarmRtM*);

	map<int,FILE*> path_log_;            // Log file handle
	map<int,float> path_last_sample_;    // Time when last sample was logged
	void log_path(O_Ants2006_SwarmRtM*);

	map<int,float> delay_avg_;
	map<int,FILE*> delay_log_; 
	void log_delay(O_Ants2006_SwarmRtM*);

	map<int,FILE*> gamma_elite_log_;            // Log file handle
	map<int,float> gamma_elite_last_sample_;    // Time when last sample was logged
	void log_gamma_elite(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,FILE*> gamma_elite_log2_;            // Log file handle
	map<int,float> gamma_elite_last_sample2_;    // Time when last sample was logged
	void log_gamma_elite2(O_Ants2006_SwarmRtM*);

	map<int,FILE*> gamma_all_log_;            // Log file handle
	map<int,float> gamma_all_last_sample_;    // Time when last sample was logged
	void log_gamma_all(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,FILE*> gamma_all_log2_;            // Log file handle
	map<int,float> gamma_all_last_sample2_;    // Time when last sample was logged
	void log_gamma_all2(O_Ants2006_SwarmRtM*);

	map<int,FILE*> gamma_diff_log_;            // Log file handle
	map<int,float> gamma_diff_last_sample_;    // Time when last sample was logged
	void log_gamma_diff(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,FILE*> gamma_diff_log2_;            // Log file handle
	map<int,float> gamma_diff_last_sample2_;    // Time when last sample was logged
	void log_gamma_diff2(O_Ants2006_SwarmRtM*);

	map<int,FILE*> gamma_rel_log_;            // Log file handle
	map<int,float> gamma_rel_last_sample_;    // Time when last sample was logged
	void log_gamma_rel(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,FILE*> gamma_rel_log2_;            // Log file handle
	map<int,float> gamma_rel_last_sample2_;    // Time when last sample was logged
	void log_gamma_rel2(O_Ants2006_SwarmRtM*);

	map<int,FILE*> elitelimit_log_;            // Log file handle
	map<int,float> elitelimit_last_sample_;    // Time when last sample was logged
	void log_elitelimit(O_Ants2006_SwarmRtM*);

	map<int, long> backtrackcount_;            // Log file accumulator 
	map<int, FILE*> backtrackcount_log_;       // Log file handle
	map<int, float> backtrackcount_last_sample_;    // Time when last sample was logged
	void log_backtrackcount(O_Ants2006_SwarmRtM*);

	map<int,long> forwardcount_;            // Log file accumulator 
	map<int,FILE*> forwardcount_log_;       // Log file handle
	map<int,float> forwardcount_last_sample_;    // Time when last sample was logged
	void log_forwardcount(O_Ants2006_SwarmRtM*);

	map<int,FILE*> rho_log_;            // Log file handle
	map<int,float> rho_last_sample_;    // Time when last sample was logged
	void log_rho(O_Ants2006_SwarmRtM*);

	map<nsaddr_t,FILE*> phero_log_;            // Log file handle
	map<nsaddr_t,float> phero_last_sample_;    // Time when last sample was logged
	map<nsaddr_t,map<nsaddr_t,map<int,double> > > phero_last_level_;    // Last phero level sampled for a pheroid
	void log_phero(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<nsaddr_t,FILE*> phero_log2_;            // Log file handle
	map<nsaddr_t,float> phero_last_sample2_;    // Time when last sample was logged
	map<nsaddr_t,map<nsaddr_t,map<int,double> > > phero_last_level2_;    // Last phero level sampled for a pheroid
	void log_phero2(O_Ants2006_SwarmRtM*);

	map<int,long> datapacketcount_;            // Log file accumulator 
	map<int,FILE*> datapacketcount_log_;       // Log file handle
	map<int,float> datapacketcount_last_sample_;    // Time when last sample was logged
	void log_datapacketcount(O_Ants2006_SwarmRtM*);

	map<int,long> datapacketreceived_;            // Log file accumulator 
	map<int,FILE*> datapacketreceived_log_;       // Log file handle
	map<int,float> datapacketreceived_last_sample_;    // Time when last sample was logged
	void log_datapacketreceived(O_Ants2006_SwarmRtM*);

	map<O_Ants2006_generator*,FILE*> genrate_log_;            // Log file handle
	map<O_Ants2006_generator*,float> genrate_last_sample_;    // Time when last sample was logged
	void log_genrate(O_Ants2006_generator*);

	map<int,FILE*> datapacketcost_log_;            // Log file handle
	map<int,float> datapacketcost_last_sample_;    // Time when last sample was logged
	void log_datapacketcost(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,FILE*> datapacketcost_log2_;            // Log file handle
	map<int,float> datapacketcost_last_sample2_;    // Time when last sample was logged
	void log_datapacketcost2(O_Ants2006_SwarmRtM*);	

	map<int,FILE*> cost_all_log_;            // Log file handle
	map<int,float> cost_all_last_sample_;    // Time when last sample was logged
	void log_cost_all(O_Ants2006_SwarmRtM*);

	//(Oriol):
	map<int,FILE*> cost_all_log2_;            // Log file handle
	map<int,float> cost_all_last_sample2_;    // Time when last sample was logged
	void log_cost_all2(O_Ants2006_SwarmRtM*);

	map<int,long> arrivedcount_;            // Log file accumulator 
	map<int,FILE*> arrivedcount_log_;       // Log file handle
	map<int,float> arrivedcount_last_sample_;    // Time when last sample was logged
	void log_arrivedcount(O_Ants2006_SwarmRtM*);
};




#endif
