// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/common/swarm-rtmodule.cc,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: Basic swam routing module
//       Holds code for both swarm node and swarm agent functionality.
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-07-06


#include <stdio.h>

#include "tcl.h"
#include "dynalink.h"
#include "swarm-rtmodule.h"
#include "classifier-swarm.h"


// Debug output macros
#define DEBUG(args)      
//#define DEBUG(args) printf args; fflush(NULL)
#define DEBUG_NEIGHBOUR(args) 
//#define DEBUG_NEIGHBOUR(args) printf args; fflush(NULL)

RNG SwarmRtModule::randomgen_;

// Tcl interface 
static class SwarmRtModuleClass : public TclClass {
public:
	SwarmRtModuleClass() : TclClass("RtModule/Swarm") {}
	TclObject* create(int argc, const char*const* argv) {
		return (new SwarmRtModule());
	}
}class_SwarmRtModule;


SwarmRtModule::SwarmRtModule() : RoutingModule(), TimerHandler() 
{  

	// Prepare neighbourhood list
	neighbourhood_.clear();

	// Set swarm type
	myswarmtype_ = SWT_NONE;
	
	// Set default swarm agent processing time
	service_time_ = 0.0;

	trace_ = NULL;
}

SwarmRtModule::~SwarmRtModule()
{
	// Clean up table of refs to neighbours
	neighbourhood_.clear();

}

int SwarmRtModule::command(int argc, char const *const *argv) {
	// Process commands fired at TCL-level

	// Access to tcl layer
	Tcl& tcl = Tcl::instance(); 
	
	if (argc == 2) {

		if (strcmp(argv[1], "unregister") == 0) {
			// Routing module is to be removed from node
      
			// Clean up
			delete classifier_; classifier_ = NULL;
			if ( unreg_route_notifications() == TCL_ERROR) return TCL_ERROR;

			return TCL_OK;
		}

	} else  if (argc == 3) {

		if (strcmp(argv[1], "register") == 0) {
			// Routing module is to be added to node
			const char* node = argv[2];

			// Attach to node and subscribe to routing notifications
			if (attach_node(node) == TCL_ERROR) return TCL_ERROR;
			//			if (reg_route_notifications(node) == TCL_ERROR) return TCL_ERROR;
			tcl.evalf ("%s route-notify %s", node, this->name());   // (Via Tcl necessarry :-( )

			// Create and install classifier
			tcl.evalf("new Classifier/Swarm %s %s", swt_info::name(myswarmtype_), this->name());
			//classifier_ = new SwarmClassifier(myswarmtype_, this);
			classifier_ = (SwarmClassifier*)TclObject::lookup(tcl.result());
			tcl.evalf("%s insert-entry %s %s 0", node, this->name(), classifier_->name());

			return TCL_OK;
		}

		if (strcmp(argv[1] , "route-notify") == 0)
			return reg_route_notifications(argv[2]);

		if (strcmp(argv[1] , "unreg-route-notify") == 0)
			return unreg_route_notifications(argv[2]);

		if (strcmp(argv[1], "seed") == 0) {
			// Seed random generator
			unsigned long s;
			if(sscanf(argv[2],"%ld", &s) == 1) {
				randomgen_.set_seed(RNG::RAW_SEED_SOURCE, s);
				return TCL_OK;
			}
			tcl.add_errorf("SwarmRtModule::command: Seeding failed.");
			return TCL_ERROR;
		}

		if (strcmp(argv[1], "service-time") == 0) {
			// Set swarm agent processing time
			if(sscanf(argv[2],"%f", &service_time_) == 1)
				return TCL_OK;

			tcl.add_errorf("SwarmRtModule::command: Invalid service time %s.", argv[2] );
			service_time_ = 0.0;
			return TCL_ERROR;
		}

		if (strcmp(argv[1], "trace") == 0) {
			// Set swarm trace agent 
			trace_ = (SwarmTrace*)TclObject::lookup(argv[2]);
			
			if(trace_)
				return TCL_OK;

			tcl.add_errorf("SwarmRtModule::command: Invalid trace agent %s.", argv[2]);
			return TCL_ERROR;
		}
		

	} else  if (argc == 4) {
		if (strcmp(argv[1], "add-route") == 0) {
			
			DEBUG(("%f From tcl: Add-route\n", now()));

			// Create copy of string to handle const char* restriction...
			char *tmp_argv2 = new char[strlen(argv[2])+1];
			strncpy(tmp_argv2, argv[2], strlen(argv[2])+1);
			
			// Potentially a new neighbour
			//addNeighbour(tmp_argv2, (NsObject*)TclObject::lookup(argv[3]));
			add_route(tmp_argv2, (NsObject*)TclObject::lookup(argv[3]));

			delete tmp_argv2;
			
			// Pass event onto next module in node (if any)
			//tcl.evalf("	if { $next_rtm_ != \"\" } { $next_rtm_ add-route $dst $target }");

			return TCL_OK;
		}
		if (strcmp(argv[1], "delete-route") == 0) {

			DEBUG(("%f From tcl: Delete-route\n", now()));

			// Create copy of string to handle const char* restriction...
			char *tmp_argv2 = new char[strlen(argv[2])+1];
			strncpy(tmp_argv2, argv[2], strlen(argv[2])+1);
			
			// Potentially a new neighbour
			//deleteNeighbour(tmp_argv2, (NsObject*)TclObject::lookup(argv[3]));
			delete_route(tmp_argv2, (NsObject*)TclObject::lookup(argv[3]));

			delete tmp_argv2;
			
			// Pass event onto next module in node (if any)
			//tcl.evalf("	if { $next_rtm_ != \"\" } { $next_rtm_ delete-route $dst $target }");

			return TCL_OK;
		}
	}   
	
	return (RoutingModule::command(argc, argv));
}

void SwarmRtModule::recv(Packet* p, Handler* h)
	// Handle incoming swarm packet
{
	if (p)
		// Queue packet
		packetqueue_.push((qelement){p,h});
	
	// Process packet if possible
	poll_queue();
	   
}

void SwarmRtModule::poll_queue() 
	// Init. processing of queued packet if possible
{
	if(status() == TIMER_IDLE && ! packetqueue_.empty()) {
		// Swarm agent "processor" is available and 
		// there are packets queued
		qelement next = packetqueue_.front();
		packetqueue_.pop();
		runSwarmAgent(next.p, next.h);
		
	}
}

void SwarmRtModule::add_route(char *dst, NsObject *target)
{
	// Potentially a new neighbour
	addNeighbour(dst, target);

	//DEBUG(("Adding route to %s...\n", dst));
	
	// Pass event onto next module in node (if any)
	if (next_rtm_ != NULL)
		next_rtm_->add_route(dst, target);
}

void SwarmRtModule::delete_route(char *dst, NsObject *nullagent)
{
	// A neighbour may have disapperad
	deleteNeighbour(dst, nullagent);

	//DEBUG(("Deleting route to %s...\n", dst));

	// Pass event onto next module in node (if any)
	if (next_rtm_)
		next_rtm_->delete_route(dst, nullagent);
}

void SwarmRtModule::runSwarmAgent(Packet* p, Handler* h)
  // "Bring-back-to-life" method
  // implementing swarm agent behavoir
{   
	// Store current packet and handler
	p_ = p;
	h_ = h;

	// Simulate swarm agent processing delay
	//	resched(service_time_);

	expire(NULL);  // No rescheduling

	// Swarm agent implementation is continued in expire();
}

void SwarmRtModule::expire(Event *)
  // Timeout handler
  // Applied to simulate prosessing time reguired by swarm agents
{
	printf("SwarmRtModule::expire: This method should be overridden.\n");

	// Process next queued packet (if any)
	poll_queue();
	
}


void SwarmRtModule::addNeighbour(const char* dst, NsObject* target) {
	// Handel reference to new neighbour node

	Connector* entry = (Connector *)target;
	nsaddr_t address = (nsaddr_t) atoi(dst);

	DEBUG_NEIGHBOUR(("%d -> %d: ", node_address_, address));
	
	Neighbour* newneighbour = new Neighbour(address, entry);

	DEBUG_NEIGHBOUR(("\n")); 
	
	if (newneighbour->oneHopAway()) {
		// The neighbour is a real neighbour, i.e. one hop away.

		// Clean up any old references to neighbour
		deleteNeighbour(dst, target);

		// Add new neighbour
		neighbourhood_.push_back( newneighbour );    // To general list
		addr2neighbour_.insert(pair<nsaddr_t, Neighbour*>(address, newneighbour ));  // In address map
		entry2neighbour_.insert(pair<NsObject*, Neighbour*>((NsObject*)entry, newneighbour ));  // In entry map

		DEBUG(("Added %d - %d: ", node_address_, address));
		for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++)
			DEBUG(("%d ", (*i)->addr()));
		DEBUG(("\n"));
				

	} else {
		// Ignore "neighbours" further than one hop away.
		delete newneighbour;
		DEBUG(("Not neighbours %d - %d\n", node_address_, address));
	}
}

void SwarmRtModule::deleteNeighbour(const char* dst, NsObject* target) {
	// Remove reference to old neighbour node

	Connector* entry = (Connector*)target;
	nsaddr_t address = (nsaddr_t)atoi(dst);

	if (neighbourhood_.empty()) return;

	//Delete from address to neighbour map (index)
	{
		pair< multimap<nsaddr_t, Neighbour*>::iterator,
			multimap<nsaddr_t, Neighbour*>::iterator > todelete = addr2neighbour_.equal_range(address);
		addr2neighbour_.erase(todelete.first, todelete.second);
	}
	//Delete from link entry to neighbour map (index)
	{
		pair< multimap<NsObject*, Neighbour*>::iterator,
			multimap<NsObject*, Neighbour*>::iterator > todelete = entry2neighbour_.equal_range((NsObject*)entry);
		entry2neighbour_.erase(todelete.first, todelete.second);
	}
		
	// Delete from general neighbour list
	for(list<Neighbour*>::iterator i = neighbourhood_.begin();
		i != neighbourhood_.end(); i++) {
		if((*i)->addr() == address && (*i)->entry() == entry) {
			// Remove neighbour
			delete (*i);
			neighbourhood_.erase(i);

			DEBUG(("Deleted %d - %d : ", node_address_, address));
			for (list<Neighbour*>::iterator i = neighbourhood_.begin(); i != neighbourhood_.end(); i++)
				DEBUG(("%d ", (*i)->addr()));
			DEBUG(("\n"));
			

			return;

		}
	}
	DEBUG(("Not found %d - %d\n", node_address_, address));
	
}

int SwarmRtModule::attach_node( const char* node) {
	// Store attachment to node

	// Access to tcl layer
	Tcl& tcl = Tcl::instance(); 

	n_ = (Node*)TclObject::lookup(node);
	if (n_ == NULL) {
		tcl.add_errorf("SwarmRtModule::attach_node: Cannot attach to node with object name %s",node);
		return TCL_ERROR;
	}

	// Get address of node
	node_address_ = n_->address();

	return TCL_OK;
}  

int SwarmRtModule::reg_route_notifications( const char* node) {
	// Send request to node for route notifications

	// Access to tcl layer
	Tcl& tcl = Tcl::instance(); 

	Node *noderef = (Node *)(TclObject::lookup(node));
	if (noderef == NULL) {
		tcl.add_errorf("SwarmRtModule::reg_route_notifications: Invalid node object %s", node);
		return TCL_ERROR;
	}
	if (noderef != n_) {
		tcl.add_errorf("SwarmRtModule::reg_route_notifications: Node object %s is not attached node", node);
		return TCL_ERROR;
	}
	n_->route_notify(this);

	return TCL_OK;
}  

int SwarmRtModule::unreg_route_notifications() {
	// Send request to node for route notifications

	// Access to tcl layer
	Tcl& tcl = Tcl::instance(); 

	if (n_ == NULL) {
		tcl.add_errorf("SwarmRtModule::unreg_route_notifications: Not attached to node.");
		return TCL_ERROR;
	}
	n_->unreg_route_notify(this);
	return TCL_OK;
}

int SwarmRtModule::unreg_route_notifications( const char* node) {
	// Send request to node for route notifications

	// Access to tcl layer
	Tcl& tcl = Tcl::instance(); 

	Node *noderef = (Node *)(TclObject::lookup(node));
	if (noderef == NULL) {
		tcl.add_errorf("SwarmRtModule::unreg_route_notifications: Invalid node object %s", node);
		return TCL_ERROR;
	}
	if (noderef != n_) {
		tcl.add_errorf("SwarmRtModule::unreg_route_notifications: Node object %s is not attached node", node);
		return TCL_ERROR;
	}
	n_->unreg_route_notify(this);
	return TCL_OK;
}  


// Lookup table with recognised link element classes
const char* Neighbour::known_linkelementclass_[] = { 
	"Connector", 
	"DynamicLink",
	"Trace", 
	"Queue", 
	"DelayLink", 
	"TTLChecker",
	"*LAST*"
};


Neighbour::Neighbour(nsaddr_t addr, NsObject* entry) { 
	// Acquire references to all elements which realize link towards neighbour node

	Tcl& tcl = Tcl::instance();

	// Store parameters
	addr_ = addr; entry_ = entry;

	// Get ref to entry of neighbour node
	tcl.evalf("[[Simulator instance] get-node-by-addr %d] entry", addr);
	NsObject* nodeentry = (NsObject*)TclObject::lookup(tcl.result());

	NsObject* linkelement = entry;
	linkelements_.clear();

	tcl.evalf("%s info class", nodeentry->name());
	const char* classname = tcl.result();
	if (classname == NULL || strncmp(classname, "Classifier", 10) != 0 ) {
		printf("Neighbour::Neighbour: Class \"%s\" of neighbour node entry not recognised.\n", 
			   (classname != NULL? classname : "no class name"));
		return;
	}

	DEBUG_NEIGHBOUR(("%ld: ", entry));  

	// Travers chain of link elements until entry of neigbour node is found
	while (linkelement != nodeentry) {
		// Get type of link element
		tcl.evalf("%s info class", linkelement->name());

		if ((classname = tcl.result())) {
			// Link element has tcl class name
			
			DEBUG_NEIGHBOUR(("%s ", classname));

			if (strncmp(classname, "Classifier", 10) == 0) {
				// Unexpected neigbour node/agent found, i.e. neighbour 
				// being handled is more than one hop away
				linkelements_.clear();

				DEBUG_NEIGHBOUR((" DROPPED"));

				return;
			}

			if (strncmp(classname, "Agent/Null", 10) == 0) {
				// Unexpected neigbour agent found, treat as none-neighbour.
				linkelements_.clear();

				DEBUG_NEIGHBOUR((" DROPPED"));

				return;
			}

			bool class_recognized = false;
			// Look through recognized link element classes
			for (int i = 0; strcmp(known_linkelementclass_[i], "*LAST*") != 0; i++)
				if (strncmp(classname, known_linkelementclass_[i], strlen(known_linkelementclass_[i])) == 0 ) {

				// Valied link element, store class name and ref
				linkelements_[known_linkelementclass_[i]] = (Connector*)linkelement;
				// Move to next element in chain
				linkelement = ((Connector *)linkelement)->target();

				class_recognized = true;

				break;
				}
			
			if (! class_recognized) {
				// Unknown element base class
				printf("Neighbour::Neighbour: Unknown link element found \"%s\".\n", classname);
				linkelements_.clear();

				DEBUG_NEIGHBOUR((" DROPPED"));

				return;
			}
		} else {
			// No class name avaiable
			printf("Neighbour::Neighbour: Class name of link element unavailabe.\n");
			linkelements_.clear();

	  			DEBUG_NEIGHBOUR((" DROPPED"));

			return;
		}      
	}

}

Neighbour::~Neighbour() {
	// Clean up
	linkelements_.clear();
}

bool Neighbour::linkdown() {
	// Return true if link to neighbour is down

	if (linkelements_.find("DynamicLink") != linkelements_.end() ) {
		// Link element exists
		// DEBUG(("DynamicLink found!\n"));
		return (((DynamicLink*)linkelements_["DynamicLink"])->status() == 0 );
	}
	// Not a dynamic link, i.e. always up
	return false;
	
	// TO BE EXTENDED....

}

//(Oriol):
double Neighbour::bandwidth() {
	// Return true if link to neighbour is down

	if (linkelements_.find("DelayLink") != linkelements_.end() ) {
		// Link element exists
		// DEBUG(("DynamicLink found!\n"));
		return (((LinkDelay*)linkelements_["DelayLink"])->bandwidth() );
	}
	// Not a dynamic link, i.e. always up
	return 0;
	
	// TO BE EXTENDED....

}

//(Oriol)
double Neighbour::traffic() {
	// Return true if link to neighbour is down

	if (linkelements_.find("DelayLink") != linkelements_.end() ) {
		// Link element exists
		// DEBUG(("DynamicLink found!\n"));
		return (((LinkDelay*)linkelements_["DelayLink"])->traffic() );
	}
	// Not a dynamic link, i.e. always up
	return 0;
	
	// TO BE EXTENDED....

}

double Neighbour::ultimtemps() {
	// Return true if link to neighbour is down

	if (linkelements_.find("DelayLink") != linkelements_.end() ) {
		// Link element exists
		// DEBUG(("DynamicLink found!\n"));
		return (((LinkDelay*)linkelements_["DelayLink"])->finish_packet_time() );
	}
	// Not a dynamic link, i.e. always up
	return 0;
	
	// TO BE EXTENDED....

}

double Neighbour::retard() {
	// Return true if link to neighbour is down

	if (linkelements_.find("DelayLink") != linkelements_.end() ) {
		// Link element exists
		// DEBUG(("DynamicLink found!\n"));
		return (((LinkDelay*)linkelements_["DelayLink"])->delay() );
	}
	// Not a dynamic link, i.e. always up
	return 0;
	
	// TO BE EXTENDED....

}
//(Oriol)
double Neighbour::bitsQueueDrT() {
	// Return true if link to neighbour is down

	if (linkelements_.find("Queue") != linkelements_.end() ) {
		// Link element exists
		// DEBUG(("DynamicLink found!\n"));
		return (((Queue*)linkelements_["Queue"])->bitsLengthAvg() );
		
	}
	// Not a dynamic link, i.e. always up
	return 0;
	
	// TO BE EXTENDED....

}
