// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/common/swarm-rtmodule.h,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: Basic swam system manager
//       Holds code for both swarm node and swarm agent functionality.
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-07-06


#ifndef ns_swarmrtmodule_h
#define ns_swarmrtmodule_h

#include <list>
#include <queue>
#include <map>

#include "config.h"
#include "scheduler.h"
#include "timer-handler.h"
#include "rng.h"
#include "rtmodule.h"
#include "connector.h"
#include "swarm.h"
#include "node.h"

// Packet queue element
typedef struct {
	Packet* p;
	Handler* h;
} qelement;
	   

// Link element structure
typedef struct {
	const char* classname;
	Connector* ref;
} lelement;

struct ltstr {
  bool operator()(const char* s1, const char* s2) const {
    return strcmp(s1, s2) < 0;
  }
};

class Neighbour {
	// Handles info related to a link connecting a neighbour node
public:
	Neighbour(nsaddr_t, NsObject *);
	~Neighbour();

	// Return true if neighbour is one hop away.
	bool oneHopAway() { return (! linkelements_.empty() ); };  
	nsaddr_t addr() { return addr_; };
	NsObject* entry() { return entry_; };

	double traffic();  //(Oriol)
	double retard();		//(Oriol)
	double ultimtemps();  //(Oriol)
	double bitsQueueDrT();	//(Oriol)
    double bandwidth(); // Return the Bandwidth of the link (Oriol)
	bool linkdown(); 	// Return true if link to neighbour is down or does not exist
	
	// Return link properties
	//  double getBandwidth();    
	//int getQueuelength();  
	//double getProbagationdelay();
	//double getSecuritylevel();

protected:

	static const char* known_linkelementclass_[];   // Lookup table with recognised link element classes

	nsaddr_t addr_;                   // Address of this neighbour
	NsObject* entry_;                 // Entry point for this neighbour

	//	list<lelement> linkelements_;     // Sequence of link elements leading to neighbour
	map<const char*, Connector*, ltstr> linkelements_;     // Sequence of link elements leading to neighbour

};

// Definitions for address->Neighbour map and entry->Neighbour map
//struct ltnsaddr_t {
//	bool operator()(nsaddr_t a1, nsaddr_t a2) const {
//		return (a1 < a2);
//  }
//};
//struct ltentry {
//	bool operator()(NsObject* o1, NsObject* o2) const {
//		return (o1 < o2);
//  }
//};



class SwarmTrace;

class SwarmRtModule : public RoutingModule, public TimerHandler {
	// Basic swarm system manager
public:
	SwarmRtModule();
	~SwarmRtModule();
	virtual const char* module_name() const { return "swarm"; }
	virtual int command(int argc, const char*const* argv);
	virtual void runSwarmAgent(Packet*, Handler* );        // "Bring-back-to-life" method
	virtual void expire(Event *);                // Timeout handler
	
	void recv(Packet*, Handler*);	        // Handle incoming swarm packet
	virtual void add_route(char *, NsObject *);     // Handle route notification
	virtual void delete_route(char *, NsObject *);  // Handle route notification

	// Random generator
	static RNG randomgen_;                    // Random generator 
	inline nsaddr_t node_address() { return node_address_; };   // Address to this node

protected:

	inline double now() { 
		return Scheduler::instance().clock();  // Current simulation time
	}
	
	int attach_node( const char*);              // Attach to node
	void addNeighbour(const char*, NsObject*);              // Handles additions to neighbourhood
	void deleteNeighbour(const char*, NsObject*);           // Handles removals to neighbourhood
	int reg_route_notifications( const char*);    // Subscribe to route notifications
	int unreg_route_notifications();              // Unsubscibe route notifications
	int unreg_route_notifications( const char*);

	SwarmType myswarmtype_;                   // Swarm type handled
	nsaddr_t node_address_;                   // Address of attached node

	list<Neighbour*> neighbourhood_;                   // List of neighbour nodes
	multimap<nsaddr_t, Neighbour*> addr2neighbour_;    // Address to neighbour lookup map
	multimap<NsObject*, Neighbour*> entry2neighbour_;  // Link entry  to neighbour lookup map

	queue<qelement> packetqueue_;             // Queue of swarm packets to be processed
	void poll_queue();                        // Init. processing of queued packet if possible

	float service_time_;                      // Swarm agent processing delay

	Packet* p_;                               // Packet currently being processed
	Handler* h_;                              // Handler for current packet

	SwarmTrace* trace_;                       // Ref to trace agent
};

#endif
