// -*-	Mode:C++; c-basic-offset:4; tab-width:4; indent-tabs-mode:t -*- */
//
// Copyright (c) Q2S, NTNU 2005 All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// 3. All advertising materials mentioning features or use of this software
//    must display the following acknowledgement:
//      This product includes software developed by Q2S, NTNU.
//
// 4. The name of Q2S, NTNU may not be used to endorse or promote products 
//    derived from this software without specific prior written permission.
//
// The software is provided "as is" without express or implied warranty of 
// any kind.
//
// These notices must be retained in any copies of any part of this software.
//
// $Header: /local/cvshome/SOURCECODE/CEants_ns-allinone-2.30/ns-2.30/swarm/common/swarm.h,v 1.1 2006/10/02 12:46:45 wittner Exp $
//
// File: Header file specifying swarm packet header and valid swarm types
// Author: Otto Wittner (wittner@q2s.ntnu.no), 2005-06-30

#ifndef ns_swarm_h
#define ns_swarm_h

#include "packet.h"

// Swarm types
enum SwarmType {
	// No-type
	SWT_NONE,    
	SWT_RandomWalk,
	SWT_CEantTSP,
	SWT_CEant,
	SWT_Ants2006,
	SWT_IWSOS2006,
	SWT_O_Ants2006,
	// ** Add new swarm types ABOVE this line  ** // 
	SWT_LAST
};

// Swarm packet header
struct hdr_swarm {
	SwarmType type_;

	static int offset_;     // offset for this header
	inline static int& offset() { return offset_; }
	inline static hdr_swarm* access(Packet* p) {
		return (hdr_swarm*) p->access(offset_);
	}

	/* per-field member functions */
	SwarmType& type() { return (type_); }
};


// String names for swarm types
class swt_info {
public:
	swt_info() {
		name_[SWT_NONE] = "none";
		name_[SWT_RandomWalk] = "RandomWalk";
		name_[SWT_CEantTSP] = "CEantTSP";
		name_[SWT_CEant] = "CEant";
		name_[SWT_Ants2006] = "Ants2006";
		name_[SWT_IWSOS2006] = "IWSOS2006";
		name_[SWT_O_Ants2006] = "O_Ants2006";
	// ** Add new swarm types-to-name mappings ABOVE this line ** // 
		name_[SWT_LAST] = "undefined";
	}
	static const char* name(SwarmType s) { 
		// Map type to name
		if ( s <= SWT_LAST ) return name_[s];
		return 0;
	}
	static SwarmType type(const char* name) {
		// Map name to type
		for(int i=0; i<=SWT_LAST; i++)
			if(strcmp(name, name_[i]) == 0)
				return (SwarmType)i;
		return SWT_NONE;
	}

	static char* name_[SWT_LAST+1];

};


// Swarm agent states
#define SWS_IDLE              0
#define SWS_FORWARDSEARCH     1
#define SWS_BACKTRACKING      2
#define SWS_PROCESS_SIM       3



#endif
